#include <vector>

// Hold LineInfo
struct LineInfo {
  TF1 *line;
  double x1;
  double x2;
};
double TransfoX(double X, vector<double> &MinX, vector<double> &MaxX,
                vector<double> &NewMinX, vector<double> &NewMaxX);

void GetBin(vector<double> &NBin, TH1F *&Hist, int StartBin, int EndBin);

void CalculateNewBin(vector<double> &MinX, vector<double> &MaxX,
                     vector<double> &NewMinX, vector<double> &NewMaxX,
                     vector<double> &NewBinWidth, vector<double> &ScalingBin,
                     vector<double> &Scale, vector<double> &NBin, TGraph *&Func,
                     int BinPerMM, double BinLowEdge, double Step);

void LinkBinsWithLines(vector<LineInfo> &lineFunctions, TH1F *&h,
                       TGraph *&GraphLine);

void Linearisation7(int RunStart, int RunStop) {
  /*
    This function will Linearize the reconstructed position in X and Y for each
    FPMW of VAMOS.

    It will parse through an histogram mm per mm and store in a vector
    a scale to apply to each event depending on their position on the
    FPMW . Then, event per event it'll apply this scale which is made to ensure
    a linear distribution of the event in the FPMW.

    Input :
      int RunStart : Index of the starting run. The run to be treated should be
      named linea_{index}.root

      int RunStop : Index of the final run.

      Root file of data

    Output :
      A root file similar to the input one with added branch with linearised
      position.
  */
  //========================================================================
  //-------------------Open tree and define Hist----------------------------
  //========================================================================

  // Import the chain
  TChain *ch = new TChain("PhysicsTree");
  for (int run = RunStart; run <= RunStop; run++) {
    ch->Add(Form("../../root/analysis/linea_%d.root", run));
  } // End of run loop

  // Declaration of the histograms storing the reconstructed position
  // We define 2 histograms storing the current position and 2 holding
  // Previously non analysed position loaded from a file.

  vector<TH1F *> Position, OldPos, VarBin, DetUnmod, DetLinea;
  vector<TGraph *> LineaGraph;
  for (int i = 1; i <= 4; i++) {
    std::string PosXName = "hPositionXDet" + std::to_string(i);
    std::string PosXTitle = "PositionXDet" + std::to_string(i);
    std::string PosYName = "hPositionYDet" + std::to_string(i);
    std::string PosYTitle = "PositionYDet" + std::to_string(i);

    std::string DetUnmodXName = "hDetXUnmod" + std::to_string(i);
    std::string DetLineaXName = "hDetXLinea" + std::to_string(i);
    std::string DetUnmodYName = "hDetYUnmod" + std::to_string(i);
    std::string DetLineaYName = "hDetYLinea" + std::to_string(i);

    TH1F *PosX;
    TH1F *PosY;
    TH1F *DetUnmodXTemp;
    TH1F *DetLineaXTemp;
    TH1F *DetUnmodYTemp;
    TH1F *DetLineaYTemp;
    int LowBin = -200, Highbin = 200 ;
    int  Nbin = (Highbin -LowBin)*10;
    int LowBin2 = -1200 ;
    int  Nbin2 = (Highbin - LowBin2)*10;

    if (i == 1) {
      PosX = new TH1F(PosXName.c_str(), PosXTitle.c_str(), Nbin, LowBin, Highbin);
      PosY = new TH1F(PosYName.c_str(), PosYTitle.c_str(), Nbin, LowBin, Highbin);
      DetUnmodXTemp =
          new TH1F(DetUnmodXName.c_str(), DetUnmodXName.c_str(), 400, 0, 40);
      DetLineaXTemp =
          new TH1F(DetLineaXName.c_str(), DetLineaXName.c_str(), 400, 0, 40);
      DetUnmodYTemp =
          new TH1F(DetUnmodYName.c_str(), DetUnmodYName.c_str(), 600, 0, 60);
      DetLineaYTemp =
          new TH1F(DetLineaYName.c_str(), DetLineaYName.c_str(), 600, 0, 60);
    } // End if det 1

    if (i == 2) {
      PosX = new TH1F(PosXName.c_str(), PosXTitle.c_str(), Nbin , LowBin, Highbin);
      PosY = new TH1F(PosYName.c_str(), PosYTitle.c_str(), Nbin, LowBin, Highbin);
      DetUnmodXTemp =
          new TH1F(DetUnmodXName.c_str(), DetUnmodXName.c_str(), 630, 0, 63);
      DetLineaXTemp =
          new TH1F(DetLineaXName.c_str(), DetLineaXName.c_str(), 630, 0, 63);
      DetUnmodYTemp =
          new TH1F(DetUnmodYName.c_str(), DetUnmodYName.c_str(), 920, 0, 92);
      DetLineaYTemp =
          new TH1F(DetLineaYName.c_str(), DetLineaYName.c_str(), 920, 0, 92);

    } // End if det 2

    if (i == 3 || i == 4) {
      PosX = new TH1F(PosXName.c_str(), PosXTitle.c_str(), Nbin2, LowBin2, Highbin);
      PosY = new TH1F(PosYName.c_str(), PosYTitle.c_str(), Nbin, LowBin, Highbin);
      DetUnmodXTemp = new TH1F(DetUnmodXName.c_str(), DetUnmodXName.c_str(),
                               10000, -1000, 0);
      DetLineaXTemp = new TH1F(DetLineaXName.c_str(), DetLineaXName.c_str(),
                               10000, -1000, 0);
      DetUnmodYTemp =
          new TH1F(DetUnmodYName.c_str(), DetUnmodYName.c_str(), 1600, 0, 160);
      DetLineaYTemp =
          new TH1F(DetLineaYName.c_str(), DetLineaYName.c_str(), 1600, 0, 160);
    } // End if det 3 or 4

    Position.push_back(PosX);
    Position.push_back(PosY);
    DetUnmod.push_back(DetUnmodXTemp);
    DetUnmod.push_back(DetUnmodYTemp);
    DetLinea.push_back(DetLineaXTemp);
    DetLinea.push_back(DetLineaYTemp);
  } // End of i loop   

  //========================================================================
  //-----------------------Importing conditional var------------------------
  //========================================================================
  TFPMWPhysics *FPMW = new TFPMWPhysics();
  ch->SetBranchAddress("FPMW", &FPMW);
  // Loading the OldPosX and OldPosY histogram
  TFile *InputHist =
      //TFile::Open("Test1.root"); // If it doesnt work import backup
      TFile::Open("HistFPMW.root"); // If it doesnt work import backup
  if (!InputHist || InputHist->IsZombie()) {
    cerr << "Error opening file !" << endl;
    return;
  }
  for (int i = 1; i <= 4; i++) {
    std::string PosXName = "hPositionXDet" + std::to_string(i);
    std::string PosXTitle = "PositionXDet" + std::to_string(i);
    std::string PosYName = "hPositionYDet" + std::to_string(i);
    std::string PosYTitle = "PositionYDet" + std::to_string(i);

    OldPos.push_back( (TH1F *)InputHist->Get(PosXName.c_str()) );
    OldPos.push_back( (TH1F *)InputHist->Get(PosYName.c_str()) );

    OldPos[2*(i - 1)]->SetDirectory(nullptr);
    OldPos[2*(i - 1)+1]->SetDirectory(nullptr); 

  } // End of i loop

  InputHist->Close();


  for (int i = 1; i <= 4; i++) {    
    if (false){
      std::string CanvasName = "test" + std::to_string(i);

      TCanvas *c = new TCanvas(CanvasName.c_str(), CanvasName.c_str(), 960, 540);
      c->Divide(2,1);
      c->cd(1);
      OldPos[2*(i-1)]->Draw("logY");
      c->cd(2);
      OldPos[2*(i-1)+1]->Draw("logY");
      gPad->SetLogy();
    }
  }

  //========================================================================
  //---------------------Calculate New Bin----------------------------------
  //========================================================================
  bool Verbose = true; // Just for some cout

  vector<vector<vector<double>>> MinDet, MaxDet, NewMinDet, NewMaxDet,
    NewBinWidthDet, ScalingBinDet, ScaleDet, NBinDet;

  for (int DetNbr = 0; DetNbr < OldPos.size(); DetNbr++) {
    cout << DetNbr << endl;
    // Declaration of all vectors necessary to hold the information
    // Outputted by the function for each part of the detector.
    vector<vector<double>> MinXAll, MaxXAll, NewMinXAll, NewMaxXAll,
      NewBinWidthAll, ScalingBinAll, ScaleAll, NBinAll;

    // This hold the Bin edges to make a control final histogram with variable
    // bin width.
    vector<double> VarBinHolder;

    // Defining some useful parameter for later.
    // For now it only works for det 0.
    int NumberBins = OldPos[DetNbr]->GetNbinsX();
    double WidthHist = OldPos[DetNbr]->GetBinLowEdge(NumberBins) +
      OldPos[DetNbr]->GetBinWidth(NumberBins) -
      OldPos[DetNbr]->GetBinLowEdge(1);
    int CurrentBin = 1;
    // Generate a function for each BinPerMM (each mm then) to transform the
    // distribution Into a non uniform distribution
    cout << WidthHist << endl;
    vector<LineInfo> LinePerMM;
    TH1F *HistBigBin = (TH1F *)OldPos[DetNbr]->Rebin(20 , "HistBigBin");
    TGraph *Graphline = new TGraph();

    LinkBinsWithLines(LinePerMM, HistBigBin, Graphline);

    double normalisation =
      double(HistBigBin->GetNbinsX()) / (double(OldPos[DetNbr]->GetNbinsX()));
      double integrale = Graphline->Integral();
      cout << integrale << endl; 
      for (int i = 0; i < Graphline->GetN(); i++) {
        double xi, yi ;
        Graphline->GetPoint(i, xi, yi);
        Graphline->SetPoint(i, xi, yi * normalisation);
      }
      integrale = Graphline->Integral();
      cout << integrale << endl;

      LineaGraph.push_back(Graphline);
      // We will now parse through all the histogram, mm per mm and define a new
      // binning and scales that will linearise this histogram.
      double Step = 1;
      int BinPerStep = NumberBins / WidthHist * Step;
      for (double mm = OldPos[DetNbr]->GetBinLowEdge(1);
          mm < OldPos[DetNbr]->GetBinLowEdge(NumberBins); mm += Step) {

        // We define vector to hold the old and new binning, the bin width of each
        // bin and the scaling to apply to the bin width and to the X as well as
        // the number of event per bin.
        size_t N = BinPerStep;
        vector<double> MinX, MaxX, NewMinX, NewMaxX, NewBinWidth, ScalingBin,
          Scale, NBin;

        // We fill Nbin with the value stored in each bin of the current range.
        GetBin(NBin, OldPos[DetNbr], CurrentBin, CurrentBin + BinPerStep);
        // Fill The vector of edges of bin and bin width and scaling.
        CalculateNewBin(MinX, MaxX, NewMinX, NewMaxX, NewBinWidth, ScalingBin,
            Scale, NBin, Graphline, BinPerStep, mm, Step);

        // Remember that there is another vector of vector holding all those
        // values
        // .
        MinXAll.push_back(MinX);
        MaxXAll.push_back(MaxX);
        NewMinXAll.push_back(NewMinX);
        NewMaxXAll.push_back(NewMaxX);
        NewBinWidthAll.push_back(NewBinWidth);
        ScalingBinAll.push_back(ScalingBin);
        ScaleAll.push_back(Scale);
        NBinAll.push_back(NBin);

        for (int i = 0; i < NewMinX.size(); i++) {
          // Filling a vector containing all the new bin edges.
          if (NewMinX.size()!= 0){
             VarBinHolder.push_back(NewMinX[i]);
          }
          if (Verbose == false) {
            cout << "MinX[i]   " << MinX[i];
            cout << "  NewMinX[i]  " << NewMinX[i] << endl;
            cout << " NBIN " << NBin[i] << endl;
          }                       // end verbose
        }                         // end loop on bin                   i
        CurrentBin += BinPerStep; // Changing Current Bin
      }                           // Loop on mm of the FPMW
      // VarBinHolder.push_back(MaxXAll[MaxXAll.size() -
      // 1][MaxXAll[MaxXAll.size()-1].size() - 1]);

      if (Verbose == false && DetNbr==0) {
        for (int i = 0; i < VarBinHolder.size(); i++) {
          cout << " Var holder " << VarBinHolder[i] << " " << endl;
          ;
        } // end loop varbin
      }   // end cout
      // Filling the VarBin Hist

      std::string VarBinName = "hVarbin" + std::to_string(DetNbr);
      VarBin.push_back(new TH1F(VarBinName.c_str(), VarBinName.c_str(), VarBinHolder.size() - 1,
           &VarBinHolder[0]));
      MinDet.push_back(MinXAll);
      MaxDet.push_back(MaxXAll);
      NewMinDet.push_back(NewMinXAll);
      NewMaxDet.push_back(NewMaxXAll);
      NewBinWidthDet.push_back(NewBinWidthAll);
      ScalingBinDet.push_back(ScalingBinAll);
      ScaleDet.push_back(ScaleAll);
      NBinDet.push_back(NBinAll);
      VarBinHolder.clear();
  } // end loop on det
  //========================================================================
  //---------------------------Treat Event----------------------------------
  //========================================================================
  // return;
  int Nentries = ch->GetEntries();
  int GoodEntries = 0;

  for (int entries = 0; entries < Nentries; entries++) {
    ch->GetEntry(entries);
    if (entries % 100000 == 0) {
      cout << "\033[92m\r Processing tree..."
        << (double)entries / Nentries * 100 << "\% done" << flush;
    } // End cout loading

    // Verification that we are not on a empty event for the FPMW
    if (FPMW->DetectorNbr.size() > 0) {
      GoodEntries += 1;
      // Parsing through the different FPMW
      for (int DetectorNbr = 0; DetectorNbr < FPMW->DetectorNbr.size();
          DetectorNbr++) {
        double X = FPMW->PositionX[DetectorNbr];
        double Y = FPMW->PositionY[DetectorNbr];

        // Just filling two sanity check histo,
        //  To delete in final version
        if (DetectorNbr >= 0 && DetectorNbr < 4) {
          Position[DetectorNbr * 2]->Fill(X);
          Position[DetectorNbr * 2 + 1]->Fill(Y);
        }

        // For detector 0 -> In final version loop on detector must be done
        // For all bin in the MinXAll vector
        int IndexX = DetectorNbr * 2;
        for (int binX = 0; binX < MinDet[IndexX].size(); binX++) {
          // If X is in the correct range
          if (MinDet[IndexX][binX].size() != 0) {
            if (X >= MinDet[IndexX][binX][0] &&
                X < MaxDet[IndexX][binX][MaxDet[IndexX][binX].size() - 1]) {
              // Apply the coorect transformation to X
              double XNew =
                TransfoX(X, MinDet[IndexX][binX], MaxDet[IndexX][binX],
                    NewMinDet[IndexX][binX], NewMaxDet[IndexX][binX]);

              // Fill of histogram for testing
              VarBin[IndexX]->Fill(XNew);
              DetUnmod[IndexX]->Fill(X);
              DetLinea[IndexX]->Fill(XNew);

              // Some cout
              if (Verbose == false) {
                cout << X << " " << XNew << "  " << MinDet[IndexX][binX][3]
                  << "  " << NewMinDet[IndexX][binX][3] << endl;
                cout << "************************" << endl;
              }
            } // end check in range
          }   // end check empty
        }     // end loop mmX

        int IndexY = DetectorNbr * 2 + 1;
        for (int binY = 0; binY < MinDet[DetectorNbr * 2 + 1].size(); binY++) {
          if (MinDet[IndexY][binY].size() != 0) {
            if (Y >= MinDet[IndexY][binY][0] &&
                Y < MaxDet[IndexY][binY][MaxDet[IndexY][binY].size() - 1]) {
              double YNew =
                TransfoX(Y, MinDet[IndexY][binY], MaxDet[IndexY][binY],
                    NewMinDet[IndexY][binY], NewMaxDet[IndexY][binY]);

              VarBin[IndexY]->Fill(YNew);
              DetUnmod[IndexY]->Fill(Y);
              DetLinea[IndexY]->Fill(YNew);

            } // end check in range
          }   // End non empty
        }     // End loop BINY
      }       // Loop detnbr
    }         // Non empty event
  }           // End loop on entries

  cout << endl << "Good entries" << GoodEntries << endl;

  //========================================================================
  //---------------------Output histogram-----------------------------------
  //========================================================================
  TFile *OutputHist = new TFile("HistFPMW.root", "RECREATE");
  std::vector<TCanvas *> canvas, cLinea;
  for (int i = 0; i < DetUnmod.size(); i++) {
    std::string CanvasName = "c" + std::to_string(i);
    std::string CanvasNameLinea = "Linearised" + std::to_string(i);
    if (true){
      TCanvas *c = new TCanvas(CanvasName.c_str(), CanvasName.c_str(), 960, 540);
      canvas.push_back(c);

      canvas[i]->cd(1);
      Position[i]->Draw("logY");
      Position[i]->Write();
      gPad->SetLogy();
    }
    TCanvas *Clin = new TCanvas(CanvasNameLinea.c_str(),
        CanvasNameLinea.c_str(), 1860, 540);
    Clin->Divide(3, 1);
    cLinea.push_back(Clin);

    cLinea[i]->cd(1);
    DetUnmod[i]->Draw();
    LineaGraph[i]->SetMarkerStyle(21);
    LineaGraph[i]->SetMarkerColor(kRed);
    LineaGraph[i]->Draw("P same");
    cLinea[i]->cd(2);
    DetLinea[i]->Draw("");
    LineaGraph[i]->SetMarkerStyle(21);
    LineaGraph[i]->SetMarkerColor(kRed);
    LineaGraph[i]->Draw("P same");
    DetUnmod[i]->Write();
    cLinea[i]->cd(3);
    VarBin[i]->Draw("");
    VarBin[i]->Write();
    gStyle->SetOptStat(11111111);
  }
  OutputHist->Close();

} // end macro

//========================================================================
//--------------------------TransfoX--------------------------------------
//========================================================================

double TransfoX(double X, vector<double> &MinX, vector<double> &MaxX,
    vector<double> &NewMinX, vector<double> &NewMaxX) {
  /*
     This take as input the original bin of the histogram and the new bin
     defined by the function CalculateNewBin And output the new calculated X

     This assume that the NewMinX and MinX have the same size !!!
     */
  double XNew = -1000;
  for (int i = 0; i < MinX.size(); i++) {
    if (X >= MinX[i] && X < MaxX[i]) {
      double Xold = X;
      double Scale2 = (X - MinX[i]) / (MaxX[i] - MinX[i]);
      XNew = Scale2 * (NewMaxX[i] - NewMinX[i]) + NewMinX[i];

      if (false) {
        cout << X << endl;
        cout << XNew << endl;
      }
      // Sanity check print
      if ((XNew < NewMinX[i] || XNew > NewMaxX[i]) && false) {
        cout << " FUNCTION  " << endl;
        cout << i << endl;
        cout << "MinX " << MinX[i] << " MaxX " << MaxX[i] << endl;
        cout << "Xold " << Xold << endl;
        cout << "NewMinX " << NewMinX[i] << " NewMaxX " << NewMaxX[i] << endl;
        cout << "Xnew  " << XNew << endl;
        cout << endl << endl;
      }
    }
  }
  return XNew;
} // end transfo x

//========================================================================
//--------------------------CalculateNewBin-------------------------------
//========================================================================

void CalculateNewBin(vector<double> &MinX, vector<double> &MaxX,
    vector<double> &NewMinX, vector<double> &NewMaxX,
    vector<double> &NewBinWidth, vector<double> &ScalingBin,
    vector<double> &Scale, vector<double> &NBin, TGraph *&Func,
    int BinPerMM, double BinLowEdge, double Step) {
  /*
     This function take as input the nimber of bin per mm and the starting
     position in the histogram and return several vector.

     Those vector old information on the edges of the bin of the initial
     histogram and the new calculated bin made for linearisation of the
     reconstructed position

     Most importantly it return SCALE which is used for event per event
     transformation of the reconstructed position in order to linearise the
     distribution for this specific range of value in this specific detector.

Input :
NBin : Number of event per bin in the range we are looking at.

Bin Per MM : Number of bin in the range we are looking at.

BinLowEdge : Current position in the histogram

ScalingBin : function associated to each bin in the histogram
NB we need to find the subset of this vector with the functions
of interest for our current position in the hist. Usually
there is 2 function per range.

Output :
Min X , MaxX : Vector of the edges of the unlinearised bin.

NewMinX, NewMaxX : Vector of the edges of the linearised bin.

Scale : Scaling to be applied to the Max edge of each bin. This is the
main output of this macro

*/

  // Define some control variable
  double SumBin = 0, TotWidth = 0., NewTotWidth = 0., SumEval = 0,
  IndexLastBin = BinPerMM, Temp = 0;
  bool MemorySave = true;
  vector<double> BinWidth, NewBinWidthTheoretical, NBinNonEmpty, ScalingBinTemp;

  // We need to find the function of interest
  double BinUpperEdge = BinLowEdge + Step;

  // Here we fill the vector for the unlinearised hist
  // Those can help to make cout to ensure that the histogram is nicely
  // imported.
  for (int i = 0; i < NBin.size(); i++) {
    double d = i;
    if (NBin[i] == 0 && MemorySave == true) {
      continue;
    }

    else {
      MinX.push_back(BinLowEdge + (d / (NBin.size())) * Step);
      MaxX.push_back(BinLowEdge + ((d + 1) / (NBin.size())) * Step);
      SumBin += NBin[i];
      TotWidth += BinWidth[i];
      NBinNonEmpty.push_back(NBin[i]);
    }
  }
  for (int i = 0; i < MinX.size(); i++) {
    BinWidth.push_back(MaxX[i] - MinX[i]);
    ScalingBinTemp.push_back(Func->Eval((MaxX[i])));
    SumEval += ScalingBinTemp[i];
  }
  double dilatation = SumBin / SumEval ;
  SumEval = SumEval * dilatation ;
  for (int i = 0 ; i<MinX.size() ; i++){
    ScalingBinTemp[i] = ScalingBinTemp[i] * dilatation;
  }
  // Loop on the bin in the mm
  int FirstIter = 1;
  for (int i = 0; i < MinX.size(); i++) {

    if (FirstIter == 1) {
      NewMinX.push_back(MinX[i]);
      FirstIter = 0;
    }

    else if (i != 0) {
      // If we are not in the first bin the NewMinEdge should start at the
      // previous max edge
      NewMinX.push_back(NewMaxX[i - 1]);
    }

    // Just initialising the first new bin edge
    // We will multiply the bin width by the following value
    // by applying a transformation to the upper edge of the bin

    ScalingBin.push_back(NBinNonEmpty[i] /
        (ScalingBinTemp[i] * (SumBin / SumEval)));

    // The new bin should have a width equal to the following value
    NewBinWidthTheoretical.push_back(ScalingBin[i] * BinWidth[i]);

    // This would be the new max without the transformation of scaling bin. aka
    // if the bin were not linearised.
    double MaxXTemp = NewMinX[i] + BinWidth[i];

    // A scaling factor that will only be applied to the max edge of the bin
    // while giving a bin width given by the NewBinWidthTheoretical is to be
    // defined as follow
    Scale.push_back((NewMinX[i] + BinWidth[i] * ScalingBin[i]) / MaxXTemp);
    NewMaxX.push_back((NewMinX[i] + BinWidth[i]) * Scale[i]);

    // NewMinX is defined as all the edges of the bin not only the lower edges
    // so we need to assugb the last element manually.

    // if (NewMaxX.size() > 0 && NewMaxX[i] > MaxX[MaxX.size() - 1] ) {
    // NewMaxX[i] = MaxX[MaxX.size() - 1];
    // }
    if (i == MaxX.size() - 1) {
      NewMaxX[i] = MaxX[i];
    }
    NewBinWidth.push_back(NewMaxX[i] - NewMinX[i]);
    NewTotWidth += NewBinWidth[i];
    if (false && (MinX[i] >= 5 && MinX[i] <= 6)) {
      // Those are sanity check cout. To be turned on as needed.
      cout << " itération " << i << endl;
      cout << " BinLowEdge " << BinLowEdge << " step " << Step
        << " IndexLastBin " << IndexLastBin << endl;
      cout << " MinX " << MinX[i] << " MaxX " << MaxX[i] << endl;
      cout << " NewMin " << NewMinX[i] << "  NewMax " << NewMaxX[i] << endl;
      cout << " ScalingBinTemp  " << ScalingBinTemp[i] << " NewBinWidth "
        << NewBinWidth[i] << endl;
      cout << " Width of bin " << NewBinWidth[i] << "  expected width  "
        << NewBinWidthTheoretical[i] << endl;
      cout << " Func Eval MaxX " << Func->Eval(MaxX[i]) << endl;
      cout << " NBin " << NBinNonEmpty[i] << " NBinOpt " << ScalingBinTemp[i]
        << endl;
      cout << "Sum Eval " << SumEval << " SumBin " << SumBin << endl;
      cout << endl << endl;
    }
  } //End loop bin
}

//========================================================================
//--------------------------GetBin----------------------------------------
//========================================================================
void GetBin(vector<double> &NBin, TH1F *&Hist, int StartBin, int EndBin) {
  /*
     Store into a input vector the value in a certain range of an histogram.
     */
  for (int bin = StartBin; bin < EndBin; bin++) {
    NBin.push_back(Hist->GetBinContent(bin));
  }
}

void LinkBinsWithLines(vector<LineInfo> &lineFunctions, TH1F *&h,
    TGraph *&GraphLine) {
  /*
     Take an histogram as input and output linear function
     linking the center of those histogram.
     */
  // Number of bins in the histogram
  int nBins = h->GetNbinsX();

  lineFunctions.clear();
  // Loop over the bins and create TF1 functions for each adjacent bin pair
  for (int i = 1; i < nBins; ++i) {
    // Get the centers of the current bin and the next bin
    double x1 = h->GetBinLowEdge(i);
    double y1 = h->GetBinContent(i);
    double x2 = h->GetBinLowEdge(i + 1);
    double y2 = h->GetBinContent(i + 1);

    // Calculate the slope and intercept of the line
    double slope = (y2 - y1) / (x2 - x1);
    double intercept = y1 - slope * x1;

    // Create the TF1 function for this line
    TF1 *line = new TF1(Form("line_%d", i), "[0]*x+[1]", x1, x2);
    line->SetParameter(0, slope);
    line->SetParameter(1, intercept);

    // Store the function in the vector
    lineFunctions.push_back({line, x1, x2});

    // Add point to graph
    GraphLine->SetPoint(GraphLine->GetN(), x1, y1);
    if (i == nBins - 1) {
      GraphLine->SetPoint(GraphLine->GetN(), x2, y2);
    }
  }
  if (true) {
    TCanvas *cline = new TCanvas("cline", "cline", 800, 600);
    h->Draw();
    for (size_t i = 0; i < lineFunctions.size(); ++i) {
      lineFunctions[i].line->SetLineColor(kRed);
      lineFunctions[i].line->Draw("same");
    } // end parser

    GraphLine->SetMarkerStyle(21);
    GraphLine->SetMarkerColor(kBlue);
    GraphLine->Draw("P same");
  } // end print
}
