# Calibration of IC
The goal of this directory is to obtain the charge of the fission fragment.
To do so we need to flatten the DE E plot and make it as precise as possible.

- All the calibration of this directory are automatically savec in Calibration/VAMOS 
- You should comment the spline line in config/ConfigIC.dat

## Step 0 : Clean calibration

Execute the ClearCalib.sh macro to clear all the calibration from the analysis.

Then run an analysis to generate a clean root file name VamosCalib$RunNumber$.root

## Step 1 : Time offset
The first step is to calculate the time offset needed to calculate the DriftTime.
To do so we focus on multiplicity two event in the MWPC and make sure that the time of those events are the same.

The macro is straightforward : put an unanalysed run in the tchain and let it generate the calibration.


## Step 2 : Correction of IC

Now that we have the DriftTime we have a very precise way to pinpoint the position in Y of the fission fragment.

The next step of the calibration is to make all the segment of the IC react the same way in Y. Expect for the IC 0 that need a X
correction as well. We will take IC1 as the benchmark. 

### IC0
IC0_XYCorrections is the macro made to correct the defects in XY. It make the ratio between IC1 and IC0 for event and then make a
profile of this 3D plot to get a surface representing the ratio between the two segments. 

After that it applies a correction to flatten the surface.

### IC2-4

The steps for the following IC is a bit more complicated. The goal is to flatten the response of each segment iteratively with
respect to the previous one.

To do so, we take IC1 as the baseline and make sucessive ration of the IC : IC[i]/IC[i-1] vs DrifTime. Then we make a spline of
those histograms.

Those spline are the one used to flatten the distribution and are the calibration that we seek from this macro.

**If you want to improve the Z resolution** : The IC 2-3 are well behaved but the 4 has a factor 2 in the analysis.
Therefore one should be very careful about the range in which the spline is made for IC4 !! 
This is a point to be improved !
The range in Y should be custom for each section of the chio. One should take time to code something to make this range finding
automatic, for instance project the ratio in a histogram then fitting it by a gaussian and take the range as mean +- 3 * sigma.

# Step 3 : Correction of DE in Y

If you are attentive you must have asked yourself : if everything is aligned on IC1 behaviour what about IC1 dependency in Y ?

This is precisely what we correct in this macro and what is save in SplineDE.
This macro also serve of a visualisation for the improvement of the DE with the previous corrections.

**This macro should be where you see improvement**

This is how you are supposed to use it : 
- First : 
```
root 'DEYcorrection.C(false,false)'
```
This will generate several histogram and most importantly the DEvs DriftTime histogram. 
When it is done make a cut on one of the charge for each DEvsDriftTime plot and save them in the "Output/CutDeCorr.root" file.
Name them CutZ CutZ0 CutZ02 CutZ023 CutZ0234, they correspond to different corrections of the IC.

- Then :
```
root 'DEYcorrection.C(true,true)'
```
**TWO TIMES IN A ROW** 
Then you should see all the histograms filled and beautiful.

**Now rerun an analysis and the IC.DE and IC.Eres should give a good vamos plot**
- You should uncomment the spline line in config/ConfigIC.dat

# Step 4 : Cut Auto for Z id

This macro is made to spare you the hassle of manually doing the cut of each of the charge.

It works by taking a VAMOS DE_E plot, applying \*-1 to the value of the histo and then parsing on the X axis by taking projection
on the Y axis on a bin width set by the user.

**Feed it using the associated HistoFiller just give it a calibrated run**

Then for each projection it search the peaks that correspond to the gap between each charges

Finally, it links those point together and drop the lines where the number of points is inferior to a threshold. Then it link
those line together to form TCUTG.

Here are the principals parameters you should care about to get good cuts :

- Step : The bin distance between each point on the X axis. Lowering this parameter make more precise cut at the cost of computing
speed.

- BinSize : Range in bin on which make the projection on the Y axis. The higher it is the more precise the peak finding will be
but be careful as it has some weird behaviour on edges !.

- sigma : Tspectrum parameter I don't understand

- Threshold : Size of the peak to be found .

This macro output the TCUTG in a root file in the output folder.

There is a version FineTuning of this macro : if you need good cut use this one but be wary that now there is two Threshold at the
beginning of the macro : ThresholdX and ThresholdY it correspond to the max distance of point on those axes. You should play
around with value until it works.

In this finetuning macro you can add manually cut at the edges of the DE_E and rename them CUT NUmber. Then by using
SortAndRenameTcutG you can rename them with the convention used by SplineChio.C and with SaveTcutGToFile you can save them.

# Step 5 : Flattening of the DE.

SplineChio.C make a spline for each cut generated in Step4 and modify each event according to it's distance to the two nearest
spline. 

The macro should be used as following :

```
root
.L SplineChio.C

SplineChio() # Only for first use after analysis

MakeSpline() # Generate the spline and save them in the Calibration file

ApplySpline() #Generate the calibration of each peak to get Z and save them in the Calibration dir
```

Don't forget to modify the configs/ConfigIC.dat with the proper token ! 
- LOAD_Z_SPLINE ./Calibration/VAMOS/CHIO/Z_Spline.root
- LOAD_Z_SPLINE_EVAL ./Calibration/VAMOS/CHIO/Chio_Z_Spline_Eval.txt

When it's done and the graphs look alright rerun the analysis.

Then you can fill and fit the histo using the associated macro to see the resolution in Z. :) 
