#include "TICPhysics.h"
#include <TChain.h>
#include <chrono>
#include <regex>



/** 
 * *  Simple macro to fill DE E histo from path
 * */
void HistoFillerDEE(const char* Path){

    //Load file
    TChain *chain = new TChain("PhysicsTree");
    chain->Add(Path);

    TICPhysics *IC = new TICPhysics();
    chain->SetBranchStatus("IC", true);
    chain->SetBranchAddress("IC", &IC); 
   
    double FF_IC_X;
    chain->SetBranchStatus("FF_IC_X", true);
    chain->SetBranchAddress("FF_IC_X", &FF_IC_X);
    
    TH2F *hDE_E = new TH2F("DE_E" , "DE_E", 1000,0,35000,1000,0,26000);
    //===========================================================================================================
    //                  Fill 
    //===========================================================================================================

    int Nentries = chain->GetEntries();
    //int Nentries = 1000;
    auto start = std::chrono::high_resolution_clock::now();

    for (int e = 0 ; e < Nentries ; e++){
        if (e%100000 ==0 && e>0){
            auto now = std::chrono::high_resolution_clock::now();
            std::chrono::duration<double> elapsed = now - start;
            double avgTimePerIteration = elapsed.count() / e ; 
            double timeLeft = avgTimePerIteration * (Nentries -e);
            std::cout << "********** Estimated time left :" << int(timeLeft) << " seconds **********" << "\r" << flush;
        } // end timer

        chain->GetEntry(e);

        if(FF_IC_X>-530 ){
            hDE_E->Fill(IC->Eres,IC->DE);
        }
    }

    std::string strPath(Path);    
    std::regex pattern("Run(\\d{3})");  // Regex to match Run followed by exactly 3 digits
    std::smatch matches;

    if (std::regex_search(strPath, matches, pattern)) {
        std::string number = matches[1];  // Extract the number
        std::string fileName = "Output/DE_E_" + number + ".root";  // Generate the filename

        // Create the TFile
        TFile* file = new TFile(fileName.c_str(), "RECREATE");
        if (file->IsOpen()) {
            std::cout << "File created: " << fileName << std::endl;
        } else {
            std::cout << "Failed to create the file." << std::endl;
        }

        hDE_E->Write();
        // Don't forget to close the file
        file->Close();
    } else {
        std::cout << "Pattern not found!" << std::endl;
    }
} // end macro
