#include "TTimeData.h"
#include <TICPhysics.h>
#include <TFPMWPhysics.h>
#include <TCanvas.h>
#include <TChain.h>
#include <TF1.h>
#include <TFile.h>
#include <TH2.h>
#include <TSpline.h>
#include <TCutG.h>
#include <TStyle.h>
#include <fstream>
#include <vector>
#include "TProfileEvaluator.h"

using namespace std;
vector<double> TxtToVector(const char *Path);
int GetNumberKey(TFile *infile ,const char* ClassName);
void TH2toCSV(TH2F* h2, const char *PathOut) ;

TSpline3* MakeSpline(TH2F* hInput, Int_t NCallee, double XMin , double
        XMax, double YMin, double YMax);
//////////////////////////////////////////////////////////////////////////////////////////
void DECorr(bool cut = false, bool spline = false) {
    //===========================================================================================================
    //                           Loading var
    //===========================================================================================================
    TChain* chain = new TChain("PhysicsTree");
    chain->Add("../../../root/analysis/VamosCalib247.root");
    //chain->Add("../../../root/analysis/Run246.root");

    TICPhysics* IC = new TICPhysics() ;
    TTimeData *Time = new TTimeData() ;
    TFPMWPhysics *FPMW = new TFPMWPhysics();

    double FF_IC_X, FF_IC_Y, FF_V13, Y, FF_Theta;

    chain->SetBranchStatus("FF_IC_X", true);
    chain->SetBranchAddress("FF_IC_X", &FF_IC_X);

    chain->SetBranchStatus("FF_IC_Y", true);
    chain->SetBranchAddress("FF_IC_Y", &FF_IC_Y);

    chain->SetBranchStatus("IC", true);
    chain->SetBranchAddress("IC", &IC); 

    chain->SetBranchStatus("FPMW", true);
    chain->SetBranchAddress("FPMW", &FPMW);   

    chain->SetBranchStatus("FF_Theta", true);
    chain->SetBranchAddress("FF_Theta", &FF_Theta);   

    chain->SetBranchStatus("FF_V13", true);
    chain->SetBranchAddress("FF_V13", &FF_V13); 

    chain->SetBranchStatus("Time", true);
    chain->SetBranchAddress("Time", &Time);

    vector<double> Toff13 , Toff14, Toff23, Toff24;
    const char* Path13 = "../../mwpc/Toff/output/Toff13.txt";
    const char* Path14 = "../../mwpc/Toff/output/Toff14.txt";
    const char* Path23 = "../../mwpc/Toff/output/Toff23.txt";
    const char* Path24 = "../../mwpc/Toff/output/Toff24.txt";

    Toff13 = TxtToVector(Path13);
    Toff14 = TxtToVector(Path14);
    Toff23 = TxtToVector(Path23);
    Toff24 = TxtToVector(Path24);

    ProfileEvaluator Profile;
    Profile.LoadProfile("Output/RatioProfile.root","ICOneZeroProfile");

    TCutG *CutZ, *CutZ0 , *CutZ02 , *CutZ023 ,*CutZ0234;
    TCutG *CutZbis;
    TFile *fCut;
    if (cut) {
        fCut =  TFile::Open("Output/CutDeCorr.root");
        CutZbis = (TCutG*)fCut->Get("CutZbis") ;
        CutZ = (TCutG*)fCut->Get("CutZ") ;
        CutZ0 = (TCutG*)fCut->Get("CutZ0") ;
        CutZ02 = (TCutG*)fCut->Get("CutZ02") ;
        CutZ023 = (TCutG*)fCut->Get("CutZ023") ;
        CutZ0234 = (TCutG*)fCut->Get("CutZ0234") ;
    }

    TFile *fSpline; 
    TSpline3 *splineDE,*splineDE0, *splineDE02,*splineDE023,*splineDE0234,*splineDEbis;
    fSpline= new TFile("Output/splineDE.root","open");
    splineDE = (TSpline3*) fSpline->Get("SplineDe");
    splineDE0 = (TSpline3*) fSpline->Get("SplineDe_0");
    splineDE02 = (TSpline3*) fSpline->Get("SplineDe_02");
    splineDE023 = (TSpline3*) fSpline->Get("SplineDe_023");
    splineDE0234 = (TSpline3*) fSpline->Get("SplineDe_0234");

    splineDEbis = (TSpline3*) fSpline->Get("SplineDebis");

    TFile *fSplineIC = new TFile("Output/spline_P2P_2024.root","open"); 
    // Get number of spline
    int SplineCount = 0 ;
    TIter next(fSplineIC->GetListOfKeys());
    TKey* key;

    while ((key=(TKey*)next())){
        if (std::string(key->GetClassName()) == "TSpline3"){
            SplineCount ++;
        }
    }
    vector<TSpline3*> spline_X(11), spline_Y(11);
    for (int i = 0; i < 11  ; i++) {
        spline_X.at(i) = (TSpline3 *)fSplineIC->Get(Form("fspline_IC%d_X",i));
        spline_Y.at(i) = (TSpline3 *)fSplineIC->Get(Form("fspline_IC%d_Y",i));
    } //End loop on histogram
      //
      
    double XmaxDE = 26000, XMaxE = 35000;
    //Defining output histo
    TH2F* hDE_E = new TH2F("DE_E","DE_E",1000,0,XMaxE,1000,0,XmaxDE);
    TH2F* hDE_E_splined = new TH2F("DE_E_splined","DE_E_splined",1000,0,XMaxE,1000,0,XmaxDE);
    TH2F* hDE_Ebis = new TH2F("DE_Ebis","DE_Ebis",1000,0,XMaxE,1000,0,24000);

    TH2F* hDE_E0 = new TH2F("DE_E0","DE_E0",1000,0,XMaxE,1000,0,XmaxDE);
    TH2F* hDE_E02 = new TH2F("DE_E02","DE_E02",1000,0,XMaxE,1000,0,XmaxDE);
    TH2F* hDE_E023 = new TH2F("DE_E023","DE_E023",1000,0,XMaxE,1000,0,XmaxDE);
    TH2F* hDE_E0234 = new TH2F("DE_E0234","DE_E0234",1000,0,XMaxE,1000,0,XmaxDE);
    TH2F* hDE_E1234 = new TH2F("DE_E1234","DE_E1234",1000,0,XMaxE,1000,0,XmaxDE);

    TH2F* hDE_Y = new TH2F("DE_Y","DE_Y",1000,-1000,1000,1000,0,XmaxDE);    
    TH2F* hDE0_Y = new TH2F("DE0_Y","DE0_Y",1000,-1000,1000,1000,0,XmaxDE);
    TH2F* hDE02_Y = new TH2F("DE02_Y","DE02_Y",1000,-1000,1000,1000,0,XmaxDE);
    TH2F* hDE023_Y = new TH2F("DE023_Y","DE023_Y",1000,-1000,1000,1000,0,XmaxDE);
    TH2F* hDE0234_Y = new TH2F("DE0234_Y","DE0234_Y",1000,-1000,1000,1000,0,XmaxDE);


    TH2F* hDE_corr_Y = new TH2F("DE_corr_Y","DE_corr_Y",500,-1000,1000,500,15000,XmaxDE);    
    TH2F* hDE0_corr_Y = new TH2F("DE0_corr_Y","DE0_corr_Y",500,-1000,1000,500,15000,XmaxDE);
    TH2F* hDE02_corr_Y = new TH2F("DE02_corr_Y","DE02_corr_Y",500,-1000,1000,500,15000,XmaxDE);
    TH2F* hDE023_corr_Y = new TH2F("DE023_corr_Y","DE023_corr_Y",500,-1000,1000,500,15000,XmaxDE);
    TH2F* hDE0234_corr_Y = new TH2F("DE0234_corr_Y","DE0234_corr_Y",500,-1000,1000,500,15000,XmaxDE);



    TH2F* hDE_Y_cut = new TH2F("DE_Y_cut","DE_Y_cut",500,-1000,1000,500,0,XmaxDE);
    TH2F* hDE0_Y_cut = new TH2F("DE0_Y_cut","DE0_Y_cut",500,-1000,1000,500,0,XmaxDE);
    TH2F* hDE02_Y_cut = new TH2F("DE02_Y_cut","DE02_Y_cut",500,-1000,1000,500,0,XmaxDE);
    TH2F* hDE023_Y_cut = new TH2F("DE023_Y_cut","DE023_Y_cut",500,-1000,1000,500,0,XmaxDE);
    TH2F* hDE0234_Y_cut = new TH2F("DE0234_Y_cut","DE0234_Y_cut",500,-1000,1000,500,0,XmaxDE);


    TH2F* hDE_Y_splined = new TH2F("DE_Ysplined","DE_Ysplined",1000,-1000,1000,1000,0,XmaxDE);
    TH2F* hDE_Y_splined_nocut = new TH2F("DE_Ysplined_nocut","DE_Ysplined_nocut",1000,-1000,1000,1000,0,22000);
    TH2F* hIC_Y = new TH2F("IC_Y","IC_Y",1000,-1000,1000,1000,0,22000);
    TH2F* hIC_Y_splined = new TH2F("IC_Ysplined","IC_Ysplined",1000,-1000,1000,1000,0,22000);


    TH2F* hDE_Y_corr = new TH2F("DE_Ycorr","DE_Ycorr",1000,-1000,1000,1000,0,22000);
    TH2F* hDE_Y_corr_nocut = new TH2F("DE_Ycorr_nocut","DE_Ycorr_nocut",1000,-1000,1000,1000,0,22000);

    TH2F* hDE_Y_bis = new TH2F("DE_Ybis","DE_Ybis",1000,-1000,1000,1000,0,22000);
    TH2F* hDEbis_Y_corr = new TH2F("DE_Ybiscorr","DE_Ybiscorr",1000,-1000,1000,1000,0,22000);

    TH2F* hDE_E_corr = new TH2F("DE_Ecorr","DE_Ecorr",1000,0,XMaxE,1000,0,XmaxDE);
    TH2F* hDE0_E_corr = new TH2F("DE0_Ecorr","DE0_Ecorr",1000,0,XMaxE,1000,0,XmaxDE);
    TH2F* hDE02_E_corr = new TH2F("DE02_Ecorr","DE02_Ecorr",1000,0,XMaxE,1000,0,XmaxDE);
    TH2F* hDE023_E_corr = new TH2F("DE023_Ecorr","DE023_Ecorr",1000,0,XMaxE,1000,0,XmaxDE);
    TH2F* hDE0234_E_corr = new TH2F("DE0234_Ecorr","DE0234_Ecorr",1000,0,XMaxE,1000,0,XmaxDE);
    TH2F* hDE_Ebis_corr = new TH2F("DE_Ebiscorr","DE_Ebiscorr",1000,0,XMaxE,1000,0,24000);

    TH2F* hDE_V = new TH2F("DE_V","DE_V",1000,0,5,1000,0,XmaxDE);

    // Def Lim spline
    double Ymin = -1000 , Ymax =1000;
    //===========================================================================================================
    //                                        Event Loop
    //===========================================================================================================
    int Nentries = chain->GetEntries();
    //int Nentries = 1000000;
    //int Nentries = 22000;
    auto start = std::chrono::high_resolution_clock::now();
    int NSegment = 11;
    ofstream debug("Output/Debug.txt");
    //save data for clustering

    for (int e = 0; e < Nentries; e++) {
        if (e % 100000  == 0 && e > 0 ) {
            auto now = std::chrono::high_resolution_clock::now();
            std::chrono::duration<double> elapsed = now - start;
            double avgTimePerIteration = elapsed.count() / e;
            double timeLeft = avgTimePerIteration * (Nentries - e);

            std::cout << "********** Estimated time left: " << int(timeLeft) << " seconds **********" << "\r" << flush;
        }

        chain->GetEntry(e);
        vector<double> ICcorr_Y(NSegment), ICcorr_X(NSegment); 
        vector<double> Temp_X(NSegment) , Temp_Y(NSegment);    



        if (Time->GetMWPC13Mult() ==1 && IC->fIC_TS.size()>=8 ){ //only mult 1 event
            UShort_t FPMW_Section = Time->GetSection_MWPC3(0);
            double FF_DriftTime =  10* (IC->fIC_TS.at(0) - Time->GetTS_MWPC13(0)) - ((Time->GetTime_MWPC13(0)+Toff13.at(FPMW_Section))) ;

            for (int seg = 1; seg < IC->fIC_TS.size() ; seg++) { // loop on multiplicity of event
                if (seg == NSegment) seg = 0; //from 1to NSeg finishing with 0

                double FF_DriftTime_temp = 10* (IC->fIC_TS.at(seg) - Time->GetTS_MWPC13(0)) - ((Time->GetTime_MWPC13(0)+Toff13.at(FPMW_Section))) ;
                if (spline_Y.at(seg)==0){
                    ICcorr_Y.at(seg) = IC->fIC_PID[seg]; 
                }

                else {
                    if (seg == 0) {
                        ICcorr_Y.at(seg) = ICcorr_Y.at(0) * spline_Y.at(0)->Eval(0)/spline_Y.at(0)->Eval(FF_DriftTime);
                    }

                    else if (seg == 1){ 
                        ICcorr_Y.at(seg) = IC->fIC_PID[1]*spline_Y.at(1)->Eval(0)/spline_Y.at(1)->Eval(FF_DriftTime);
                    }

                    else if (seg > 1) {
                        ICcorr_Y.at(seg) = IC->fIC_PID[seg] * spline_Y.at(seg)->Eval(0)/ spline_Y.at(seg)->Eval(FF_DriftTime);
                    }
                    if (!(ICcorr_Y.at(seg)==ICcorr_Y.at(seg))) ICcorr_Y.at(seg) = 0;
                } //end if non empty
                if (seg == 0) break;
            }//endloop seg



            //Double_t PolX =  -3.10262 * 1e-7 *FF_IC_X*FF_IC_X - 7.02108 * 1e-5 * FF_IC_X + 1.37622;
            Double_t PolX =  1.37622;

            Double_t ICRatio =  IC->fIC_PID[1]/IC->fIC_PID[0];
            Double_t ICRatioCorr =  ICRatio * PolX / Profile.Evaluate(FF_IC_X,FF_DriftTime,false);
            Double_t ICcorr_Y0 = IC->fIC_PID[0] / PolX * Profile.Evaluate(FF_IC_X,FF_DriftTime,false);

            if ( ICRatioCorr<1.4 || ICRatioCorr >1.5){
                ICRatioCorr =  ICRatio * PolX / Profile.Evaluate(FF_IC_X,FF_DriftTime,false);
                ICcorr_Y0 =     IC->fIC_PID[0] / PolX * Profile.Evaluate(FF_IC_X,FF_DriftTime,false);
                if (ICRatioCorr >100) {
                    ICcorr_Y0 = IC->fIC_PID[0];
                }
            }

            double DE = 0 , DE_splined=0 ,DE_corr=0, DE_Ybis=0;
            double E =0, E_splined=0;

            for (int seg = 0 ;  seg < sizeof(IC->fIC_PID)/sizeof(IC->fIC_PID[0]) ; seg++ ){
                if (seg >4){
                    if (seg == 5) E += double(IC->fIC_PID[seg]);
                    else E +=double(IC->fIC_PID[seg]) ;
                }
            }



            // ********************* DE setter *********************
            double IC1corr = (IC->fIC_PID[1]*(1-0.000686068*FF_IC_Y))*(1-4.88238e-05*FF_IC_Y+7.40395e-06*FF_IC_Y*FF_IC_Y);
            double DE_Bis = (IC1corr+IC->fIC_PID[2]+IC->fIC_PID[3])+IC->fIC_PID[4];

            DE =           ( IC->fIC_PID[0]+ double(IC->fIC_PID[1]) +double(IC->fIC_PID[2])+ double(IC->fIC_PID[3])) + double(IC->fIC_PID[4]);
            DE_splined =    ( ICcorr_Y0+ ICcorr_Y.at(1) +IC->fIC_PID[2]+ IC->fIC_PID[3]) + IC->fIC_PID[4];
            double DE0 =    ( ICcorr_Y0 + ICcorr_Y.at(1) +IC->fIC_PID[2] + IC->fIC_PID[3]) + IC->fIC_PID[4];
            double DE02 =   ( ICcorr_Y0+ ICcorr_Y.at(1) +ICcorr_Y.at(2)+ IC->fIC_PID[3]) + IC->fIC_PID[4];
            double DE023 =  ( ICcorr_Y0+ ICcorr_Y.at(1) +ICcorr_Y.at(2)+ ICcorr_Y.at(3)) + IC->fIC_PID[4];
            double DE0234 = ( ICcorr_Y0+ ICcorr_Y.at(1) +ICcorr_Y.at(2)+ ICcorr_Y.at(3)) + ICcorr_Y.at(4);
            double DE1234 = ( double(IC->fIC_PID[1]) +double(IC->fIC_PID[2])+ double(IC->fIC_PID[3])) + double(IC->fIC_PID[4]);
            //DE_splined = 0.5*( ICcorr_Y[1] +IC->fIC_PID[2]+IC->fIC_PID[3])+IC->fIC_PID[4];


            //************* Cut ******************
            bool CutV = abs( E-6000)<500;
            bool CutVbis =  abs( E-6000)<500;

            //bool CutV = true;
            //bool CutVbis =  true;

            //***********Fill histo*************

            if (/*FF_DriftTime >Ymin && FF_DriftTime <Ymax && (FF_Theta >0)*/ true){

                //*************Init DE_E***************** 
                hDE_E         -> Fill(E,DE);
                hDE_E_splined -> Fill(E,DE_splined);
                hDE_E0        -> Fill(E,DE0);
                hDE_E02       -> Fill(E,DE02);
                hDE_E023      -> Fill(E,DE023);
                hDE_E0234     -> Fill(E,DE0234);
                hDE_E1234     -> Fill(E,DE1234);

                hDE_Ebis->Fill(E,DE_Bis);
                hDE_V->Fill(FF_V13,DE_splined);    

                //******* Coor DE ************
                if (spline== true  ){     
                    double DE_corr = DE * splineDE->Eval(0) / splineDE->Eval(FF_DriftTime);
                    double DE0_corr = DE0 * splineDE0->Eval(0) / splineDE0->Eval(FF_DriftTime);
                    double DE02_corr = DE02 * splineDE02->Eval(0) / splineDE02->Eval(FF_DriftTime);
                    double DE023_corr = DE023 * splineDE023->Eval(0) / splineDE023->Eval(FF_DriftTime);
                    double DE0234_corr = DE0234 * splineDE0234->Eval(0) / splineDE0234->Eval(FF_DriftTime);

                    for(int i =0 ; i<5 ; i++){
                        if (i==0) debug << ICcorr_Y0 << "  ";
                        else debug << ICcorr_Y.at(i) << "  " ;
                    }
                    debug << IC->fIC_PID[4];
                    debug << endl;

                    hDE_E_corr->Fill(E,DE_corr);
                    hDE0_E_corr->Fill(E,DE0_corr);
                    hDE02_E_corr->Fill(E,DE02_corr);
                    hDE023_E_corr->Fill(E,DE023_corr);
                    hDE0234_E_corr->Fill(E,DE0234_corr);

                    if (CutV){
                        hDE_corr_Y->Fill(FF_DriftTime,DE_corr);
                        hDE0_corr_Y->Fill(FF_DriftTime,DE0_corr);
                        hDE02_corr_Y->Fill(FF_DriftTime,DE02_corr);
                        hDE023_corr_Y->Fill(FF_DriftTime,DE023_corr);
                        hDE0234_corr_Y->Fill(FF_DriftTime,DE0234_corr);
                    }


                    DE_Ybis = DE_Bis * splineDEbis->Eval(0) / splineDEbis->Eval(FF_DriftTime);
                    hDE_Ebis_corr->Fill(E,DE_Ybis); 

                }   

                else {
                    DE_corr = DE_splined;

                    DE_Ybis = DE_Bis ;
                    hDE_Ebis_corr->Fill(E,DE_Ybis);  
                }


                if (cut == true ) {
                    bool CutCharge     =  CutZ->IsInside(FF_DriftTime,DE);
                    bool CutCharge0     =  CutZ0->IsInside(FF_DriftTime,DE0);
                    bool CutCharge02    =  CutZ02->IsInside(FF_DriftTime,DE02);
                    bool CutCharge023   =  CutZ023->IsInside(FF_DriftTime,DE023);
                    bool CutCharge0234  =  CutZ0234->IsInside(FF_DriftTime,DE0234);


                    if (CutV ) {
                        hDE_Y_corr_nocut->Fill(FF_DriftTime,DE_corr);
                        hDE_Y_splined_nocut->Fill(FF_DriftTime,DE_splined);
                        hDE_Y->Fill(FF_DriftTime,DE);
                        hDE_Y_splined->Fill(FF_DriftTime,DE_splined); 
                        hDE_Y_corr->Fill(FF_DriftTime,DE_corr);

                        hIC_Y->Fill(FF_DriftTime,IC->fIC_PID[0]);
                        hIC_Y_splined->Fill(FF_DriftTime,ICcorr_Y0);


                        hDE_Y->Fill(FF_DriftTime,DE);
                        hDE0_Y->Fill(FF_DriftTime,DE0);
                        hDE02_Y->Fill(FF_DriftTime,DE02);
                        hDE023_Y->Fill(FF_DriftTime,DE023);
                        hDE0234_Y->Fill(FF_DriftTime,DE0234);



                        if (CutCharge)  hDE_Y_cut->Fill(FF_DriftTime,DE);
                        if (CutCharge0)  hDE0_Y_cut->Fill(FF_DriftTime,DE0);
                        if (CutCharge02)  hDE02_Y_cut->Fill(FF_DriftTime,DE02);
                        if (CutCharge023)  hDE023_Y_cut->Fill(FF_DriftTime,DE023);
                        if (CutCharge0234)  hDE0234_Y_cut->Fill(FF_DriftTime,DE0234);
                    }
                }//end cut
                else {     
                    if(true ){ 
                        hDE_Y_bis->Fill(FF_DriftTime,DE_Bis);
                        hDE_Y_splined->Fill(FF_DriftTime,DE_splined); 
                        hDE_Y_corr->Fill(FF_DriftTime,DE_corr);

                        hIC_Y->Fill(FF_DriftTime,IC->fIC_PID[0]);
                        hIC_Y_splined->Fill(FF_DriftTime,ICcorr_Y0);

                        if (CutV ) {
                            hDE_Y->Fill(FF_DriftTime,DE);
                            hDE0_Y->Fill(FF_DriftTime,DE0);
                            hDE02_Y->Fill(FF_DriftTime,DE02);
                            hDE023_Y->Fill(FF_DriftTime,DE023);
                            hDE0234_Y->Fill(FF_DriftTime,DE0234);

                        }

                    }
                }

                if (cut == true && CutVbis) {
                    hDEbis_Y_corr->Fill(Y,DE_Ybis);
                }
                else{;}
                //end cut
            }//end cond Y
        } // End mult 1
    } //endl loop event

    // making spline

    TSpline3  *splineDe, *splineDe0, *splineDe02,*splineDe023,*splineDe0234,*osplineDEbis;
    splineDe = MakeSpline(hDE_Y_cut,0,Ymin,Ymax,0,23000); 
    splineDe0 = MakeSpline(hDE0_Y_cut,1,Ymin,Ymax,0,23000); 
    splineDe02 = MakeSpline(hDE02_Y_cut,2,Ymin,Ymax,0,23000); 
    splineDe023 = MakeSpline(hDE023_Y_cut,3,Ymin,Ymax,0,23000); 
    splineDe0234 = MakeSpline(hDE0234_Y_cut,4,Ymin,Ymax,0,23000); 
    osplineDEbis = MakeSpline(hDE_Y,5,Ymin,Ymax,0,23000); 

    TFile *fSplineDE = new TFile("Output/splineDE.root","recreate");

    splineDe->SetName("SplineDe");
    splineDe->Write();

    splineDe0->SetName("SplineDe_0");
    splineDe0->Write();

    splineDe02->SetName("SplineDe_02");
    splineDe02->Write();

    splineDe023->SetName("SplineDe_023");
    splineDe023->Write();

    splineDe0234->SetName("SplineDe_0234");
    splineDe0234->Write();

    osplineDEbis->SetName("SplineDebis");
    osplineDEbis->Write(); 
    fSplineDE->Close();

    //output calibration
    TFile *fSplineDEY = new TFile("../../../Calibration/VAMOS/CHIO/Spline_DE.root","recreate");
    splineDe0234->SetName("fspline_1");
    splineDe0234->Write();
    fSplineDEY->Close();

    //===========================================================================================================
    //                                          Drawing histo
    //===========================================================================================================

    gStyle->SetPalette(kRainBow);
    // =========== Poster plot=============================

    /*
       TCanvas* c8 = new TCanvas("DE Before correction","DE Before correction",4000,4000); 
       hDE_E->GetXaxis()->SetTitle("E (A.U)");
       hDE_E->GetXaxis()->SetRangeUser(3000,35000);
       hDE_E->GetXaxis()->CenterTitle();

       hDE_E->GetYaxis()->SetTitle("#Delta E (A.U)");
       hDE_E->GetYaxis()->SetRangeUser(6000,26000);
       hDE_E->GetYaxis()->CenterTitle();

       hDE_E->GetZaxis()->SetRangeUser(0,140);
       hDE_E->GetZaxis()->SetTitle("Number of events");
       hDE_E->GetZaxis()->CenterTitle();

       hDE_E->SetTitle("PID VAMOS before correction");
       hDE_E->SetStats(0);
       hDE_E->Draw("COLZ");

       c8->SetCanvasSize(4000,3000);
       c8->SaveAs("~/These/Plot/Poster/VamosPIDBeforeCorr.png");
       c8->SaveAs("~/These/Plot/Poster/VamosPIDBeforeCorr.pdf");
       c8->SaveAs("~/These/Plot/Poster/VamosPIDBeforeCorr.jpg");

       TCanvas* c9 = new TCanvas("DE after correction","DE after correction",1000,1000);

       hDE0234_E_corr->GetXaxis()->SetTitle("E (A.U)");
       hDE0234_E_corr->GetXaxis()->SetRangeUser(3000,35000);
       hDE0234_E_corr->GetXaxis()->CenterTitle();

       hDE0234_E_corr->GetYaxis()->SetTitle("#Delta E (A.U)");
       hDE0234_E_corr->GetYaxis()->SetRangeUser(6000,26000);
       hDE0234_E_corr->GetYaxis()->CenterTitle();

       hDE0234_E_corr->GetZaxis()->SetRangeUser(0,140);
       hDE0234_E_corr->GetZaxis()->SetTitle("Number of events");
       hDE0234_E_corr->GetZaxis()->CenterTitle();

       hDE0234_E_corr->SetTitle("PID VAMOS after correction");
       hDE0234_E_corr->SetStats(0);
       hDE0234_E_corr->Draw("COLZ");

       c9->SetCanvasSize(4000,3000);
       c9->SaveAs("~/These/Plot/Poster/VamosPIDAfterCorr.png");
       c9->SaveAs("~/These/Plot/Poster/VamosPIDAfterCorr.pdf");
       c9->SaveAs("~/These/Plot/Poster/VamosPIDAfterCorr.jpg");
    //gStyle->SetOptStat(1);
    // =========== Poster plot=============================
    */


    TCanvas* c7 = new TCanvas("correction succesive","correction succesive",1500,1000);
    c7->Divide(6);

    c7->cd(1); 
    hDE_E->Draw();

    c7->cd(2); 
    hDE_E0->GetXaxis()->SetTitle("E");
    hDE_E0->GetYaxis()->SetTitle("DE");
    hDE_E0->Draw();

    c7->cd(3); 
    hDE_E02->GetXaxis()->SetTitle("E");
    hDE_E02->GetYaxis()->SetTitle("DE");
    hDE_E02->Draw();

    c7->cd(4); 
    hDE_E023->GetXaxis()->SetTitle("E");
    hDE_E023->GetYaxis()->SetTitle("DE");
    hDE_E023->Draw();

    c7->cd(5); 
    hDE_E0234->GetXaxis()->SetTitle("E");
    //hDE_E0234->GetYaxis()->SetTitle("DE");
    hDE_E0234->Draw();

    c7->cd(6); 
    hDE_E1234->GetXaxis()->SetTitle("E");
    hDE_E1234->GetYaxis()->SetTitle("DE");
    hDE_E1234->Draw();



    TCanvas *c4 = new TCanvas("c4","c4",1500,1000);
    c4->Divide(5);
    c4->cd(1);
    hDE_Y->Draw();

    c4->cd(2);
    hDE0_Y->Draw();

    c4->cd(3);
    hDE02_Y->Draw();

    c4->cd(4);
    hDE023_Y->Draw();

    c4->cd(5);
    hDE0234_Y->Draw();

    TCanvas *c4bis = new TCanvas("c4bis","c4bis",1500,1000);
    c4bis->Divide(5);
    c4bis->cd(1);
    hDE_corr_Y->Draw();

    c4bis->cd(2);
    hDE0_corr_Y->Draw();

    c4bis->cd(3);
    hDE02_corr_Y->Draw();

    c4bis->cd(4);
    hDE023_corr_Y->Draw();

    c4bis->cd(5);
    hDE0234_corr_Y->Draw();




    TCanvas* c2 = new TCanvas("c2","c2",1500,1000);
    c2->Divide(6 );

    c2->cd(1);
    gPad->Modified();
    gPad->Update(); 
    hDE_E_corr->Draw("colz");
    hDE_E_corr->GetXaxis()->SetTitle("E");
    hDE_E_corr->GetYaxis()->SetTitle("DE");

    c2->cd(2);
    gPad->Modified();
    gPad->Update();
    hDE0_E_corr->Draw("colz");
    hDE0_E_corr->GetXaxis()->SetTitle("E");
    hDE0_E_corr->GetYaxis()->SetTitle("DE");

    c2->cd(3);
    gPad->Modified();
    gPad->Update();
    hDE02_E_corr->Draw("colz");
    hDE02_E_corr->GetXaxis()->SetTitle("E");
    hDE02_E_corr->GetYaxis()->SetTitle("DE");

    c2->cd(4);
    gPad->Modified();
    gPad->Update();
    hDE023_E_corr->Draw("colz");
    hDE023_E_corr->GetXaxis()->SetTitle("E");
    hDE023_E_corr->GetYaxis()->SetTitle("DE");

    c2->cd(5);
    gPad->Modified();
    gPad->Update();
    hDE0234_E_corr->Draw("colz");

    c2->cd(6);
    hDE_Ebis->SetTitle("online");
    gPad->Modified();
    gPad->Update();
    hDE_Ebis->Draw("colz");
    hDE_Ebis->GetXaxis()->SetTitle("E");
    hDE_Ebis->GetYaxis()->SetTitle("DE");


    //===========================================================================================================
    //						Save for further analysis
    //===========================================================================================================

    TFile *oSaved = new TFile("Output/DE_E.root","recreate");
    hDE0234_E_corr->Write();
    TH2toCSV(hDE0234_E_corr,"Output/DE_E.csv");

} // End spline chio XY


int GetNumberKey(TFile *infile ,const char* ClassName){
    // Get number of spline
    int KeyCount = 0 ;
    TIter next(infile->GetListOfKeys());
    TKey* keyFit;

    while ((keyFit=(TKey*)next())){
        if (std::string(keyFit->GetClassName()) == ClassName){
            KeyCount ++;
        }
    }

    return KeyCount;

}

TSpline3* MakeSpline(TH2F* hInput, Int_t NCallee, double XMin = 0 , double
        XMax = 1000 , double YMin=0 , double YMax = 100) {

    TSpline3* gspline;

    TH1F* hProfile;
    TH1F* pfx; // extended hProfile

    TCanvas* canfit = new TCanvas(Form("CanSpline_%02d",NCallee),
            Form("canfit_%02d",NCallee), 0, 0, 1000, 500);


    //===========================================================================================================
    //                                            Init profile
    //===========================================================================================================

    // Create the TProfile
    hProfile = (TH1F*)hInput->ProfileX();
    hProfile->SetLineWidth(2);
    hProfile->SetDirectory(0);
    canfit->cd();

    hProfile->GetYaxis()->SetRangeUser(YMin, YMax);
    hProfile->Draw();
    //===========================================================================================================
    //                                          First and last bin to get to 6k
    //                                          event get promoted
    //===========================================================================================================
    int FirstBin, LastBin;
    double Treshold = 0;
    for (int bin =1 ; bin<hProfile->GetNbinsX(); bin++) {
        FirstBin = bin;
        if (hProfile->GetBinLowEdge(bin)> XMin) break;
    }
    for (int bin = hProfile->GetNbinsX(); bin>1 ; bin--) {
        LastBin = bin;
        if (hProfile->GetBinLowEdge(bin)< XMax) break;
    }
    //===========================================================================================================
    //                                          Init Extended profile function
    //===========================================================================================================
    // Create the extended TProfile
    pfx = new TH1F(
            Form("pfx_%02d", Int_t(NCallee/2)), Form("pfx_%02d", Int_t(NCallee/2)), hProfile->GetNbinsX(), hProfile->GetBinLowEdge(1),
            hProfile->GetBinLowEdge(hProfile->GetNbinsX()) + hProfile->GetBinWidth(hProfile->GetNbinsX()));
    pfx->SetLineColor(8);
    pfx->SetDirectory(0);
    //===========================================================================================================
    //                                           Fill extended profile
    //===========================================================================================================

    // fill the extended TProfile
    float newval, lastval, lasterr;
    for (int bin = 1; bin <= FirstBin; bin++) {
        newval = 0;
        pfx->SetBinContent(bin, newval);
    }


    for (int bin = FirstBin; bin <= LastBin; bin++) {
        newval = hProfile->GetBinContent(bin);
        if (newval != 0) {
            pfx->SetBinContent(bin, newval);
            pfx->SetBinError(bin, hProfile->GetBinError(bin));
            lastval = newval;
            lasterr = hProfile->GetBinError(bin);
        }
        else {
            pfx->SetBinContent(bin, lastval);
            pfx->SetBinError(bin, lasterr);
        }
    }
    pfx->Draw("same");

    gspline = new TSpline3(pfx);
    gspline->SetName(Form("fspline_%d", NCallee + 1));

    return gspline;

} // end makespline

vector<double> TxtToVector(const char *Path){
    string line;
    vector<double> values;
    ifstream file(Path); 

    if (file.is_open()) {
        while (std::getline(file, line)) {
            try {
                values.push_back(std::stod(line));
            } catch (const std::invalid_argument& e) {
                std::cerr << "Invalid number in line: " << line << '\n';
            }
        }
        file.close();
    } else {
        std::cerr << "Error opening file.\n";
    }

    return values;
}

// Example: Export 2D data for external clustering
void TH2toCSV(TH2F* h2, const char *PathOut) {
    std::ofstream outFile(PathOut);
    outFile << "E,DE,content\n"; // CSV header
    for (int i = 1; i <= h2->GetNbinsX(); ++i) {
        for (int j = 1; j <= h2->GetNbinsY(); ++j) {
            double content = h2->GetBinContent(i, j);
            if (content > 0) {
                double x = h2->GetXaxis()->GetBinCenter(i);
                double y = h2->GetYaxis()->GetBinCenter(j);
                outFile << x << "," << y << "," << content << std::endl;
            }
        }
    }
    outFile.close();
}
