#include <TICPhysics.h>
#include <TCanvas.h>
#include <TChain.h>
#include <TF1.h>
#include <TFile.h>
#include <TH2.h>
#include <TSpline.h>
#include <fstream>
#include <vector>

using namespace std;

int GetNumberKey(TFile *infile ,const char* ClassName);
//////////////////////////////////////////////////////////////////////////////////////////
void ICCorr() {
    //===========================================================================================================
    //                           Loading var
    //===========================================================================================================
    TChain* chain = new TChain("PhysicsTree");
    chain->Add("../../../root/analysis/VamosCalib241.root");

    TICPhysics* IC = new TICPhysics() ;
    double FF_IC_X, FF_IC_Y;

    chain->SetBranchStatus("FF_IC_X", true);
    chain->SetBranchAddress("FF_IC_X", &FF_IC_X);
    chain->SetBranchStatus("FF_IC_Y", true);
    chain->SetBranchAddress("FF_IC_Y", &FF_IC_Y);
    chain->SetBranchStatus("IC", true);
    chain->SetBranchAddress("IC", &IC); 


    TFile* fHisto =  TFile::Open("Output/HistoP2P.root");
    TFile* fSpline =  TFile::Open("Output/spline_P2P_2024.root");
    //TFile* fSpline =  TFile::Open("Output/spline_ICALL_2024.root");
    TFile* fFit = TFile::Open("Output/FitIC.root");

    //Defining output histo
    double NbinX = 500 , NbinY =100;
    TH2F* hDE_E = new TH2F("DE_E","DE_E",NbinX,9000,16000,NbinY,11000,18000);
    TH2F* hDE_Ecorr1 = new TH2F("DE_Ecorr1","DE_Ecorr1",NbinX,9000,16000,NbinY,11000,18000);
    TH2F* hDEBis_Ecorr_IC1 = new
        TH2F("DE_Ecorr_IC1","DE_Ecorr_IC1",NbinX,9000,16000,NbinY,11000,18000);
    TH2F* hDE_E_CorrAll = new
        TH2F("hDE_E_CorrAll","hDE_E_CorrAll",NbinX,9000,16000,NbinY,11000,18000);


    // Filling the input histo 
    int NSegment = 5 ;
    vector<TH2F*> hICseg_ICprev, hICX,hICY ,hICseg_ICprevX,hICseg_ICprevY, hICcorr;
    for(int seg=0; seg<NSegment; seg++){

        if(seg==1){
            hICX.push_back((TH2F*)fHisto->Get(Form("hChio_IC%d_X",seg)));
            hICY.push_back((TH2F*)fHisto->Get(Form("hChio_IC%d_Y",seg)));
            hICcorr.push_back(new TH2F(Form("hICorr%d_Y",seg),Form("hICorr%d_Y",seg),1000,-100,100,500,2000,9500));
        }
        else if (seg == 0){
            hICX.push_back((TH2F*)fHisto->Get(Form("hChio_IC%d_IC%d_X",seg+1,seg)));
            hICY.push_back((TH2F*)fHisto->Get(Form("hChio_IC%d_IC%d_Y",seg+1,seg)));
            hICcorr.push_back(new TH2F(Form("hICorr%d_Y",seg),Form("hICorr%d_Y",seg),1000,-100,100,500,0,2));
        }
        else {
            hICX.push_back((TH2F*)fHisto->Get(Form("hChio_IC%d_IC%d_X",seg,seg-1)));
            hICY.push_back((TH2F*)fHisto->Get(Form("hChio_IC%d_IC%d_Y",seg,seg-1)));
            hICcorr.push_back(new TH2F(Form("hICorr%d_Y",seg),Form("hICorr%d_Y",seg),1000,-100,100,500,0,2));
        }
    }

    //===========================================================================================================
    //                                            GETSPLINE
    //===========================================================================================================
    // Get number of spline
    int SplineCount = 0 ;
    TIter next(fSpline->GetListOfKeys());
    TKey* key;

    while ((key=(TKey*)next())){
        if (std::string(key->GetClassName()) == "TSpline3"){
            SplineCount ++;
        }
    }


    vector<TSpline3*> spline_X(11), spline_Y(11);
    for (int i = 0; i < SplineCount  ; i++) {
        int seg = int((i-2)/2 +1 );

        if (i < 2){
            spline_X.at(0) = (TSpline3 *)fSpline->Get("fspline_IC0_X");
            spline_Y.at(0) = (TSpline3 *)fSpline->Get("fspline_IC0_Y");
        }

        else if ( i>=2 && i<4){ 
            spline_X.at(1) = (TSpline3 *)fSpline->Get("fspline_IC1_X");
            spline_Y.at(1) = (TSpline3 *)fSpline->Get("fspline_IC1_Y");
        }
        else if (seg >=2 && (i%2 == 0) ) {
            spline_X.at(seg) = (TSpline3 *)fSpline->Get(Form("fspline_IC%d_X",seg));
        }
        else if (seg >=2 && !(i%2 == 0) ) {	
            spline_Y.at(seg) = (TSpline3 *)fSpline->Get(Form("fspline_IC%d_Y",seg));
        }
    } //End loop on histogram
      //===========================================================================================================
      //                                 get fit
      //===========================================================================================================

    int FitCount = GetNumberKey(fFit,"TF1"); 


    vector<TF1*> Fit;
    for (int i = 0; i < FitCount  ; i++) {
        Fit.push_back((TF1*)fFit->Get(Form("Fit_Y_Expo_IC%d",i)));
    } //End loop on Fit 


    //===========================================================================================================
    //                                        Event Loop
    //===========================================================================================================
    int Nentries = chain->GetEntries();
    auto start = std::chrono::high_resolution_clock::now();
    for (int e = 0; e < Nentries; e++) {

        if (e % 100000  == 0 && e > 0) {
            auto now = std::chrono::high_resolution_clock::now();
            std::chrono::duration<double> elapsed = now - start;
            double avgTimePerIteration = elapsed.count() / e;
            double timeLeft = avgTimePerIteration * (Nentries - e);

            std::cout << "  ****** Estimated time left: " << int(timeLeft) << " seconds ******" << "\r" << flush;
        }

        chain->GetEntry(e);


        vector<double> ICcorr_Y(NSegment), ICcorr_X(NSegment); // the [0] element of spline is the
        vector<double> Temp_X(NSegment) , Temp_Y(NSegment);    // correction on the first
                                                               // segment of ic
        for (int seg = 1; seg < NSegment+1 ; seg++) {

            if (seg == NSegment) seg = 0; //from 1to NSeg finishing with 0


            if (spline_Y.at(seg)==0){
                ICcorr_Y.at(seg) = IC->fIC_raw[seg]; 
            }

            else {
                if (seg == 0) {
                    Temp_Y.at(seg) = ICcorr_Y.at(1) / IC->fIC_raw[seg] * spline_Y.at(seg)->Eval(0)/ spline_Y.at(seg)->Eval(FF_IC_Y);
                    ICcorr_Y.at(seg) = ICcorr_Y.at(1) / Temp_Y.at(seg);
                }

                else if (seg == 1){ 
                    ICcorr_Y.at(seg) = IC->fIC_raw[1]*spline_Y.at(1)->Eval(0)/spline_Y.at(1)->Eval(FF_IC_Y);
                }

                else if (seg > 1) {
                    Temp_Y.at(seg) =  IC->fIC_raw[seg]/ICcorr_Y.at(seg-1) * spline_Y.at(seg)->Eval(0)/ spline_Y.at(seg)->Eval(FF_IC_Y);
                    ICcorr_Y.at(seg) = ICcorr_Y.at(seg-1) * Temp_Y.at(seg);
                }

                if (!(ICcorr_Y.at(seg)==ICcorr_Y.at(seg))) ICcorr_Y.at(seg) = 0;
            } //end if non empty
            if (seg == 0) break;
        }//endloop seg

        for (int i = 0; i < ICcorr_Y.size()  ; i++) {
            if (i ==1) hICcorr.at(i)->Fill(FF_IC_Y,ICcorr_Y.at(i));
            else hICcorr.at(i)->Fill(FF_IC_Y,Temp_Y.at(i));
        } //End loop on histogram

        double DE = 0 ,E =0 ,DE_IC1_corrY =0, DE_ICALL_corrY=0, Ecorr=0;

        for (int seg = 0 ;  seg < sizeof(IC->fIC_raw)/sizeof(IC->fIC_raw[0]) ; seg++ ){
            if (seg >4){
                E += IC->fIC_raw[seg] ;
                //Ecorr += ICcorr_Y.at(seg) ;
            }
        }

        double IC1corr = (IC->fIC_raw[1]*(1-0.000686068*FF_IC_Y))*(1-4.88238e-05*FF_IC_Y+7.40395e-06*FF_IC_Y*FF_IC_Y);
        double DE_Bis = 0.5*(IC1corr+IC->fIC_raw[2]+IC->fIC_raw[3])+IC->fIC_raw[4];

        DE = 0.5*(IC->fIC_raw[1] +IC->fIC_raw[2]+IC->fIC_raw[3])+IC->fIC_raw[4];
        DE_IC1_corrY = 0.5*(ICcorr_Y[1] +IC->fIC_raw[2]+IC->fIC_raw[3])+IC->fIC_raw[4];
        DE_ICALL_corrY = 0.5*(ICcorr_Y[0]+ICcorr_Y[1] +ICcorr_Y[2]+ICcorr_Y[3])+ICcorr_Y[4];
        double Ealt = E + IC->fIC_raw[4];

        if (FF_IC_Y >-60 && FF_IC_Y <60){
            hDE_E->Fill(E,DE);
            hDE_Ecorr1->Fill(E,DE_Bis);
            hDEBis_Ecorr_IC1->Fill(E,DE_IC1_corrY);
            hDE_E_CorrAll->Fill(E,DE_ICALL_corrY);
        }
    } //endl loop event
      //===========================================================================================================
      //                                          Drawing histo
      //===========================================================================================================

    TCanvas* c1 = new TCanvas("c1","c1",10000,1000);
    c1->Divide(NSegment,2);
    for (int i = 0 ; i<NSegment ;i++){
        c1->cd(i+1);
        hICcorr.at(i)->ProfileX()->Draw();

        c1->cd(NSegment+i+1);
        hICY.at(i)->ProfileX()->Draw();
    }


    TCanvas* c2 = new TCanvas("c2","c2",1500,1000);
    c2->Divide(2,2);
    c2->cd(1);
    //gPad->SetLogz();
    hDE_E->Draw();
    c2->cd(3);
    //gPad->SetLogz();
    hDE_Ecorr1->Draw();
    c2->cd(4);
    //gPad->SetLogz();
    hDEBis_Ecorr_IC1->Draw();
    c2->cd(2);
    //gPad->SetLogz();
    hDE_E_CorrAll->Draw();
} // End spline chio XY


int GetNumberKey(TFile *infile ,const char* ClassName){
    // Get number of spline
    int KeyCount = 0 ;
    TIter next(infile->GetListOfKeys());
    TKey* keyFit;

    while ((keyFit=(TKey*)next())){
        if (std::string(keyFit->GetClassName()) == ClassName){
            KeyCount ++;
        }
    }

    return KeyCount;

}
