#include <vector>

void GetBin(vector<double> &NBin, TH1F *&Hist, int StartBin, int EndBin);

void CalculateNewBin(vector<double> &EdgeBin, vector<double> &NewEdgeBin,
    vector<double> &NewBinWidth, vector<double> &ScalingBin,
    vector<double> &Scale, vector<double> &NBin, TGraph *&Func,
    int BinPerMM, double BinLowEdge, double Step);

void LinkBinsWithLines(TH1F *&h, TGraph *&GraphLine);

void saveVectorsToFile(const vector<vector<vector<double>>>& vec1, const vector<vector<vector<double>>>& vec2, const string& filename) ;

void LinearisationTxt() {
  // **************************************************************************
  // This macro linearize the reconstructed position in X and Y for each FPMW.
  //
  // It will parse through the histogram inputed and store in a vector
  // new edges of bin needed to smooth the distribution according to a
  // predefined graph.
  //
  // Then the old and new bin edges are stored in a YAML file which is to be
  // used for the event per event treatment in the LinearisationTransfo.cxx
  // macro.
  //
  // Input :
  //    HistoUnlinea.root : The reconstructed position histogram generated using
  //    
  //    HistoPosFiller.cxx
  //
  //
  // Output:
  //    Linearisation.txt : A txt file containing the old and new bin edges
  //    for each detector.
  // **************************************************************************

  //========================================================================
  //-------------------Open tree and define Hist----------------------------
  //========================================================================

  vector<TH1F *> OldPos, VarBin;
  vector<TGraph *> LineaGraph;
  bool Verbose = true; // Just for some cout

  // Loading the OldPosX and OldPosY histogram
  TFile *InputHist =
    TFile::Open("HistUnlinea.root"); // If it doesnt work import backup

  if (!InputHist || InputHist->IsZombie()) {
    cerr << "Error opening file !" << endl;
    return;
  }
  for (int i = 1; i <= 4; i++) {
    std::string PosXName = "hPositionXDet" + std::to_string(i);
    std::string PosXTitle = "PositionXDet" + std::to_string(i);
    std::string PosYName = "hPositionYDet" + std::to_string(i);
    std::string PosYTitle = "PositionYDet" + std::to_string(i);

    OldPos.push_back((TH1F *)InputHist->Get(PosXName.c_str()));
    OldPos.push_back((TH1F *)InputHist->Get(PosYName.c_str()));

    OldPos[2 * (i - 1)]->SetDirectory(nullptr);
    OldPos[2 * (i - 1) + 1]->SetDirectory(nullptr);
  } // End of i loop
  InputHist->Close();

  //========================================================================
  //---------------------Calculate New Bin----------------------------------
  //========================================================================

  // Initialising the output vector
  vector<vector<vector<double>>> EdgeBinDet, NewEdgeBinDet, NewBinWidthDet,
    ScalingBinDet, ScaleDet, NBinDet;

  // Parsing though each detector.
  for (int DetNbr = 0; DetNbr < OldPos.size(); DetNbr++) {

    // ************************************************************************
    //                          Initialisation
    // ************************************************************************

    // Initialasing output vector for this det
    vector<vector<double>> EdgeBinAll, NewEdgeBinAll, NewBinWidthAll,
      ScalingBinAll, ScaleAll, NBinAll;
    vector<double> VarBinHolder;

    // Defining some useful parameter for later.
    int NumberBins = OldPos[DetNbr]->GetNbinsX();
    double WidthHist = OldPos[DetNbr]->GetBinLowEdge(NumberBins) +
      OldPos[DetNbr]->GetBinWidth(NumberBins) -
      OldPos[DetNbr]->GetBinLowEdge(1);
    int CurrentBin = 1;

    // ************************************************************************
    //                          Generate Graph
    // ************************************************************************

    TGraph *GraphlineTemp = new TGraph();
    // We can rebin the histogram of input but it makes no difference. Here we
    // do it for testing purposes.
    std::string HistBigBinName = "HistBigBinDet" + std::to_string(DetNbr);

    TH1F *HistBigBin = (TH1F *)OldPos[DetNbr]->Rebin(NumberBins/40, HistBigBinName.c_str());
    LinkBinsWithLines(HistBigBin, GraphlineTemp);

    //copy hist
    TGraph *Graphline = new TGraph(*GraphlineTemp);

    // We renormalize the graph so that the eval make sense.
    double normalisation =
      double(HistBigBin->GetNbinsX()) / (double(OldPos[DetNbr]->GetNbinsX()));
    double integrale = Graphline->Integral();
    for (int i = 0; i < Graphline->GetN(); i++) {
      double xi, yi;
      Graphline->GetPoint(i, xi, yi);
      Graphline->SetPoint(i, xi, yi * normalisation);
    }
    integrale = Graphline->Integral();
    LineaGraph.push_back(Graphline);

    // ************************************************************************
    //                          Calculate New Bin in a step
    // ************************************************************************

    // We will now parse through all the histogram.
    double Step = 1;
    int BinPerStep = NumberBins / WidthHist * Step;

    for (double mm = OldPos[DetNbr]->GetBinLowEdge(1);
        mm < OldPos[DetNbr]->GetBinLowEdge(NumberBins); mm += Step) {

      // We define vector for this step
      size_t N = BinPerStep;
      vector<double> EdgeBin, NewEdgeBin, NewBinWidth, ScalingBin, Scale, NBin;

      // We fill Nbin with the value stored in each bin of the current range.
      GetBin(NBin, OldPos[DetNbr], CurrentBin, CurrentBin + BinPerStep);
      CalculateNewBin(EdgeBin, NewEdgeBin, NewBinWidth, ScalingBin, Scale, NBin,
          Graphline, BinPerStep, mm, Step);
      // We push this in the mother vector only if the bin is not empty.
      if (EdgeBin.size() != 0) {
        EdgeBinAll.push_back(EdgeBin);
        NewEdgeBinAll.push_back(NewEdgeBin);
        NewBinWidthAll.push_back(NewBinWidth);
        ScalingBinAll.push_back(ScalingBin);
        ScaleAll.push_back(Scale);
        NBinAll.push_back(NBin);
      }

      EdgeBin.clear();
      NewEdgeBin.clear();
      NewBinWidthAll.clear();
      ScalingBin.clear();
      Scale.clear();
      NBin.clear();


      // Just fill the VarBinHolder it serves for debugging purpose.
      for (int i = 0; i < NewEdgeBin.size(); i++) {
        if (NewEdgeBin.size() != 0) {
          VarBinHolder.push_back(NewEdgeBin[i]);
          if (Verbose == false) {
            cout << "EdgeBin[i]   " << EdgeBin[i];
            cout << "  NewEdgeBin[i]  " << NewEdgeBin[i] << endl;
            cout << " NBIN " << NBin[i] << endl;
          }
        }                       // end verbose
      }                         // end loop on bin
      CurrentBin += BinPerStep; // Changing Current Bin
    }                           // Loop on mm of the FPMW

    // ************************************************************************
    //                          Sanity check print
    // ************************************************************************

    if (Verbose == false && DetNbr == 0) {
      for (int i = 0; i < VarBinHolder.size(); i++) {
        cout << " Var holder " << VarBinHolder[i] << " " << endl;
      } // end loop varbin
    }   // end cout
        // Filling the VarBin Hist

        // ************************************************************************
        //                          Push vector of detector
        // ************************************************************************

    std::string VarBinName = "hVarbin" + std::to_string(DetNbr);
    VarBin.push_back(new TH1F(VarBinName.c_str(), VarBinName.c_str(),
          VarBinHolder.size() - 1, &VarBinHolder[0]));

    if (EdgeBinAll.size() != 0) {
      EdgeBinDet.push_back(EdgeBinAll);
      NewEdgeBinDet.push_back(NewEdgeBinAll);
      NewBinWidthDet.push_back(NewBinWidthAll);
      ScalingBinDet.push_back(ScalingBinAll);
      ScaleDet.push_back(ScaleAll);
      NBinDet.push_back(NBinAll);
    }
    VarBinHolder.clear();
  } // end loop on det

  // ************************************************************************
  //                          Write in TXT
  // ********************************************* ***************************

  saveVectorsToFile(EdgeBinDet,NewEdgeBinDet,"Output/Linearisation.txt");

} // end macro

//========================================================================
//--------------------------CalculateNewBin-------------------------------
//========================================================================

void CalculateNewBin(vector<double> &EdgeBin, vector<double> &NewEdgeBin,
    vector<double> &NewBinWidth, vector<double> &ScalingBin,
    vector<double> &Scale, vector<double> &NBin, TGraph *&Func,
    int BinPerMM, double BinLowEdge, double Step) {
  // ************************************************************************
  // Calculate the new edges of bin in the range to linearize the histogram.
  //
  // Input :
  //    NBin : contain the number of event in each bin
  //
  //    BinLowEdge : Starting point in the histogram
  //
  //    BinPerMM : Number of bin we want to generate the new bin for, also the
  //    number of bin per step.
  //
  //    Func : A Tgraph containing a sample of point from the distribution you
  //    want to go to.
  //
  // Output :
  //    EdgeBin : The edges of the bin of the original histogram. NB : this
  //    function doesn't take as input the original histogram, therefore those
  //    bin are generated by a simple iterative method. Thus the code is a
  //    bit flimsy to change in the histogram binning. This can be improved by
  //    taking the edges directly from the histogram.
  //
  //    NewEdgeBin : The edges of the new bin calculated to make the histogram
  //    look like func.
  //
  // Other :
  //    All the other parameter are for testing purpose. Most of them have self
  //    explanatory name.
  // ************************************************************************

  // ************************************************************************
  //                          Initialise variable
  // ************************************************************************

  bool MemorySave = false;
  double SumBin = 0, TotWidth = 0., NewTotWidth = 0., SumEval = 0,
         IndexLastBin = BinPerMM, Temp = 0;
  vector<double> MaxX, NewMaxX, BinWidth, NewBinWidthTheoretical, NBinNonEmpty,
    ScalingBinTemp;
  double BinUpperEdge = BinLowEdge + Step;

  // ************************************************************************
  //                          Fill unlinearised vector
  // ************************************************************************
  for (int i = 0; i < NBin.size(); i++) {

    // If the bin is empty do not fill the vector

    if (NBin[i] <= 0 && MemorySave == true) {
      continue;
    }
    // For now EdgeBin only contain the lower edge and is of the same size as
    // the other vector. We will make it 1 item bigger by adding the final
    // edge of the bin.
    else {
      double d = i;
      NBinNonEmpty.push_back(NBin[i]);
      EdgeBin.push_back(BinLowEdge + (d / (NBin.size())) * Step);
      MaxX.push_back(BinLowEdge + ((d + 1) / (NBin.size())) * Step);

      // Control variable
      SumBin += NBin[i];
      TotWidth += BinWidth[i];
    } // end if non empty
  }   // end loop on bin

  if (NBinNonEmpty.size() == 0 ){ return ; }
  // ************************************************************************
  //                    Initialisation of the scaling
  // ************************************************************************
  // Here we fill ScalinBinTemp which will hold the evaluation of our input
  // function for the maximum of the current bin. It is the basis for the
  // transformation of the bin.

  for (int i = 0; i < NBinNonEmpty.size(); i++) {

    // This will be the basis for the construction of the new bin
    // It is the number of event that should be in the bin.
    ScalingBinTemp.push_back(Func->Eval((MaxX[i])));

    // We fill some control vector useful for debugging
    BinWidth.push_back(MaxX[i] - EdgeBin[i]);
    SumEval += ScalingBinTemp[i];

    // And add the final element of the EdgeBin vector
    if (i == MaxX.size() - 1) {
      EdgeBin.push_back(MaxX[i]);
    } // End if last element
  }   // End loop on non empty bin

  // ************************************************************************
  //                   Normalisation of Scaling Bin Temp
  // ************************************************************************
  // Scaling bin temp come from a function that has not the same integral as our
  // distribution on the current range. We need to normalise the function to
  // conserve the integral.
  double dilatation = SumBin / SumEval;
  SumEval = SumEval * dilatation;

  for (int i = 0; i < NBinNonEmpty.size(); i++) {
    ScalingBinTemp[i] = ScalingBinTemp[i] * dilatation;
  }

  // ************************************************************************
  //                        Calculate the new bin
  // ************************************************************************

  int FirstIter = 1;
  for (int i = 0; i < NBinNonEmpty.size(); i++) {
    // We want to keep the edges of the bin as the original histogram
    // Therefore we must enforce 1st and last iteration value.
    if (FirstIter == 1) {
      NewEdgeBin.push_back(EdgeBin[i]);
      FirstIter = 0;
    }

    else if (i != 0) {
      NewEdgeBin.push_back(NewMaxX[i - 1]);
    }

    // Dilatation to apply to the bin width is defined here.
    ScalingBin.push_back(NBinNonEmpty[i] /
        (ScalingBinTemp[i] * (SumBin / SumEval)));
    NewBinWidthTheoretical.push_back(ScalingBin[i] * BinWidth[i]);

    // Max unlinearised
    double MaxXTemp = NewEdgeBin[i] + BinWidth[i];

    // Calculate the transformation to apply to the upper edge of the bin.
    Scale.push_back((NewEdgeBin[i] + BinWidth[i] * ScalingBin[i]) / MaxXTemp);
    NewMaxX.push_back((NewEdgeBin[i] + BinWidth[i]) * Scale[i]);

    // Enforce value of last bin.
    if (i == NBinNonEmpty.size() - 1) {
      NewMaxX[i] = MaxX[i];
      NewEdgeBin.push_back(MaxX[i]);
    }

    // Some control vector filling
    NewBinWidth.push_back(NewMaxX[i] - NewEdgeBin[i]);
    NewTotWidth += NewBinWidth[i];

    // Cout
    if (false && (NewEdgeBin[i] >= -2 && NewEdgeBin[i] <= -1)) {
      // Those are sanity check cout. To be turned on as needed.
      cout << " itération " << i << endl;
      cout << " BinLowEdge " << BinLowEdge << " step " << Step
        << " IndexLastBin " << IndexLastBin << endl;
      cout << " MinX " << EdgeBin[i] << " MaxX " << MaxX[i] << endl;
      cout << " NewMin " << NewEdgeBin[i] << "  NewMax " << NewMaxX[i] << endl;
      cout << " ScalingBinTemp  " << ScalingBinTemp[i] << " NewBinWidth "
        << NewBinWidth[i] << endl;
      cout << " Width of bin " << NewBinWidth[i] << "  expected width  "
        << NewBinWidthTheoretical[i] << endl;
      cout << " Func Eval MaxX " << Func->Eval(MaxX[i]) << endl;
      cout << " NBin " << NBinNonEmpty[i] << " NBinOpt " << ScalingBinTemp[i]
        << endl;
      cout << "Sum Eval " << SumEval << " SumBin " << SumBin << endl;
      cout << endl << endl;
    } // End cout
  }   // End loop bin
} // End CalculateNewBin

//========================================================================
//--------------------------GetBin----------------------------------------
//========================================================================
void GetBin(vector<double> &NBin, TH1F *&Hist, int StartBin, int EndBin) {
  // **************************************************************************
  // Fetch the number of event of the bin of a histogram in a certain range and
  // store them in a vector.
  // **************************************************************************

  for (int bin = StartBin; bin < EndBin; bin++) {
    NBin.push_back(Hist->GetBinContent(bin));
  } // End loop range
} // End GetBin

void LinkBinsWithLines(TH1F *&h, TGraph *&GraphLine) {
  // **************************************************************************
  // Generate a graph of point centered on each bin of an histogram.
  // **************************************************************************

  int nBins = h->GetNbinsX();

  for (int i = 1; i < nBins; ++i) {
    // Get the centers of the current bin and the next bin
    double x1 = h->GetBinLowEdge(i);
    double y1 = h->GetBinContent(i);
    double x2 = h->GetBinLowEdge(i + 1);
    double y2 = h->GetBinContent(i + 1);

    // Add point to graph
    GraphLine->SetPoint(GraphLine->GetN(), (x1+x2)/2, y1);
    if (i == nBins - 1) {
      GraphLine->SetPoint(GraphLine->GetN(), x2, y2);
    }
  }
  if (true) {
    std::string CanvasNameLinea = "Linearised" + std::to_string(nBins);
    TCanvas *cline = new TCanvas(h->GetName(), h->GetName(), 800, 600);
    h->Draw();
    GraphLine->SetMarkerStyle(21);
    GraphLine->SetLineStyle(1);
    GraphLine->SetMarkerColor(kRed);
    GraphLine->SetLineColor(kRed);
    GraphLine->Draw("LP same");
  } // end print
} // End LinkBinsWithLines


void saveVectorsToFile(const vector<vector<vector<double>>>& vec1, const vector<vector<vector<double>>>& vec2, const string& filename) {
  ofstream outFile(filename);
  if (!outFile.is_open()) {
    cerr << "Unable to open file for writing: " << filename << endl;
    return;
  }

  // Save first vector
  outFile << "Vector 1:\n";
  for (size_t detIdx = 0; detIdx < vec1.size(); ++detIdx) {
    outFile << "Det " << detIdx + 1 << ":\n";
    for (size_t binIdx = 0; binIdx < vec1[detIdx].size(); ++binIdx) {
      outFile << "  Bin " << binIdx << ": ";
      for (double edge : vec1[detIdx][binIdx]) {
        outFile << edge << " ";
      }
      outFile << "\n";
    }
    outFile << "\n";
  }
  outFile << "\n";  // Separate the two vectors

  // Save second vector
  outFile << "Vector 2:\n";
  for (size_t detIdx = 0; detIdx < vec2.size(); ++detIdx) {
    outFile << "Det " << detIdx + 1 << ":\n";
    for (size_t binIdx = 0; binIdx < vec2[detIdx].size(); ++binIdx) {
      outFile << "  Bin " << binIdx << ": ";
      for (double edge : vec2[detIdx][binIdx]) {
        outFile << edge << " ";
      }
      outFile << "\n";
    }
    outFile << "\n";
  }

  outFile.close();
}
