TChain *chain;
double* parameter;
//const int nb_parameter = 29;
const int nb_parameter = 24;
int iteration = 0;
int m_telescope;
int NumericalMinimization(const char* minName = "Minuit", const char* algoName = "");
double ConstantFactor(const double* parameter);

NPL::Reaction* elastic;
NPL::EnergyLoss C12C;
NPL::EnergyLoss C12Al;
NPL::EnergyLoss C12Si;
NPL::EnergyLoss U238C;
//NPL::Reaction* elasticBe;

TH2F* hEx;
TH2F* hExmean_Tel;
TH2F* hExmean_ThetaLab;
//TH2F* hExBe;
//TH2F* hExmeanBe_Tel;
//TH2F* hExmeanBe_ThetaLab;
TH1D* hExmean;

TPISTAPhysics* PISTA;
double EBeam = 5.95877*238;
double TargetThickness = 0.44*micrometer;
//double deg = 180/3.1415;
TGraph* C12CQuick = new TGraph("QuickEloss/ELossC.dat");
TGraph* C12SiQuick = new TGraph("QuickEloss/ELossSi.dat");
TGraph* C12AlQuick = new TGraph("QuickEloss/ELossAl.dat");
 
///////////////////////////////////////////////////
void Minimization2(int telescope=3){
   
    //Energy loss table
    C12C = EnergyLoss("/home/efremovt/nptool/nptool_v3/Projects/pista_e850_main/ana_e850/EnergyLossTable/C12_C.G4table","G4Table",100);
    C12Si = EnergyLoss("/home/efremovt/nptool/nptool_v3/Projects/pista_e850_main/ana_e850/EnergyLossTable/C12_Si.G4table","G4Table",100);
    C12Al = EnergyLoss("/home/efremovt/nptool/nptool_v3/Projects/pista_e850_main/ana_e850/EnergyLossTable/C12_Al.G4table","G4Table",100);
    U238C = EnergyLoss("/home/efremovt/nptool/nptool_v3/Projects/pista_e850_main/ana_e850/EnergyLossTable/U238_C.G4table","G4Table",100);

    //PISTA 
    PISTA = new TPISTAPhysics();
    NPL::InputParser parser("../../../pista_e850_2024.detector");
    PISTA->ReadConfiguration(parser);

 

    //Now we slow the beam by half of the target thickness


    //Define reaction
    elastic   = new NPL::Reaction("238U(12C,12C)238U@1418"); //put here ebeam
    EBeam = U238C.Slow(EBeam,TargetThickness/2,0) ;
    elastic->SetBeamEnergy(EBeam);

    

    //Define output hist
    hEx = new TH2F("hEx","hEx",6000,0,6000,500,-10,10);
    hExmean_Tel = new TH2F("hExmean_Tel","hExmean_Tel",8,1,9,500,-10,10);
    hExmean_ThetaLab = new TH2F("hExmean_ThetaLab","hExmean_ThetaLab",300,15,75,500,-10,10);
    /* hExBe = new TH2F("hExBe","hExBe",6000,0,6000,500,-10,10); */
    /* hExmeanBe_Tel = new TH2F("hExmeanBe_Tel","hExmeanBe_Tel",8,1,9,500,-10,10); */
    /* hExmeanBe_ThetaLab = new TH2F("hExmeanBe_ThetaLab","hExmeanBe_ThetaLab",300,15,75,500,-10,10); */
    m_telescope = telescope;

    //Input tree
    chain = new TChain("tree"); 
    //chain->Add("SelectTree.root");
    chain->Add("SelectRun/SelectTree_run235.root");

    //Buffer of 21 param corresponding to 3*8 telescope
    //We parametrise the calibration by a 2nd order polynomial
    double buffer[nb_parameter];
    for(int i=0; i<7; i++){
        buffer[3*i] = 0;
        buffer[3*i+1] = 0.0029;
        buffer[3*i+2] = 0.;
    }
    //Last tree buffer are for the xt yt zt
    buffer[nb_parameter - 1 ] = 0 ;
    buffer[nb_parameter - 2 ] = 0 ;
    buffer[nb_parameter - 3 ] = 0 ;


    parameter = new double[nb_parameter];
    parameter = buffer;
    
   
    ConstantFactor(parameter);
    NumericalMinimization("Minuit2","Migrad");


    TCanvas* c1 = new TCanvas("c1","c1",1600,800);
    c1->Divide(2,1);
    c1->cd(1);
    hExmean_Tel->Draw("colz");
    c1->cd(2);
    hExmean_ThetaLab->Draw("colz");


    /* c3->cd(1); */
    /* hExmeanBe_Tel->Draw("colz"); */
    /* c3->cd(2); */
    /* hExmeanBe_ThetaLab->Draw("colz"); */

    TCanvas* c2 = new TCanvas("c2","c2",1600,800);
    c2->Divide(2,1);
    c2->cd(1);
    hEx->Draw("colz");
    /* c2->cd(2); */
    /* hExBe->Draw("colz"); */
}



////////////////////////////////////////////////////
int NumericalMinimization(const char* minName, const char* algoName){

    ROOT::Math::Minimizer* min = ROOT::Math::Factory::CreateMinimizer(minName, algoName);

    min->SetMaxFunctionCalls(1000000);
    min->SetMaxIterations(100000);
    min->SetTolerance(0.001);
    min->SetPrecision(0.0001);
    min->SetPrintLevel(1);

    ROOT::Math::Functor f(&ConstantFactor,nb_parameter);

    min->SetFunction(f);

    for(int i=0; i<7; i++){

        int indice = 3*i;
        if(i==6) i=7;
        string par0_name = "p0_E" + to_string(i+1);
        string par1_name = "p1_E" + to_string(i+1);
        string par2_name = "p2_E" + to_string(i+1);

        min->SetLimitedVariable(indice,par0_name,0,0.01,-0.2,0.2);
        min->SetLimitedVariable(indice+1,par1_name,0.0029,0.0001,0.9*0.0029,1.1*0.0029);
        min->SetLimitedVariable(indice+2,par2_name,0,0.0001,-1.e-8,1.e-8);
    }
    min->SetLimitedVariable(24-3,"Xt",0,0.01,-2,2);
    min->SetLimitedVariable(25-3,"Yt",0,0.01,-2,2);
    min->SetLimitedVariable(26-3,"Zt",0,0.01,-1,1);
    //min->SetLimitedVariable(27-3,"ThetaX",0,0.01,-0.5,0.5);
    //min->SetLimitedVariable(28-3,"ThetaY",0,0.01,-0.5,0.5);

    min->Minimize();

    const double* xs = min->X();

    ofstream ofile;
    string filename = "PISTA_BACK_E_min.cal";
    ofile.open(filename.c_str());

    for(int i=0; i<7; i++){
        int indice = 3*i;
        if(i==6) i=7;
        string token = "PISTA_T" + to_string(i+1) + "_BACK_E";

        ofile << token << " " << xs[indice] << " " << xs[indice+1] << " " << xs[indice+2] << endl;
    }
    ofile.close();

    ofstream ofile2;
    string filename2 = "AnalysisConfig_min.dat";
    ofile2.open(filename2.c_str());
    ofile2 <<  "AnalysisConfig" << endl;
    ofile2 <<  " XTARGET_OFFSET " << xs[24-3] << endl;
    ofile2 <<  " YTARGET_OFFSET " << xs[25-3] << endl;
    ofile2 <<  " ZTARGET_OFFSET " << xs[26-3] << endl;
    //ofile2 <<  " BEAM_THETAX " << xs[27-3] << endl;
    //ofile2 <<  " BEAM_THETAY " << xs[28-3] << endl;
    ofile2.close();

    cout << "MinValue = " << min->MinValue() << endl;
    ConstantFactor(xs);

    return 0;

}

////////////////////////////////////////////////////
double ConstantFactor(const double* parameter){
    
    ROOT::EnableImplicitMT();
    iteration++;

    double DeltaE;
    double Eres;
    double Xcalc;
    double Ycalc;
    double Zcalc;
    double XTarget;
    double YTarget;
    int Telescope;
    int run_number;

    chain->SetBranchStatus("DeltaE","true");
    chain->SetBranchAddress("DeltaE",&DeltaE);

    chain->SetBranchStatus("Eres","true");
    chain->SetBranchAddress("Eres",&Eres);

    chain->SetBranchStatus("XTarget","true");
    chain->SetBranchAddress("XTarget",&XTarget);

    chain->SetBranchStatus("YTarget","true");
    chain->SetBranchAddress("YTarget",&YTarget);

    chain->SetBranchStatus("Xcalc","true");
    chain->SetBranchAddress("Xcalc",&Xcalc);

    chain->SetBranchStatus("Ycalc","true");
    chain->SetBranchAddress("Ycalc",&Ycalc);

    chain->SetBranchStatus("Zcalc","true");
    chain->SetBranchAddress("Zcalc",&Zcalc);

    chain->SetBranchStatus("Telescope","true");
    chain->SetBranchAddress("Telescope",&Telescope);

    chain->SetBranchStatus("run_number","true");
    chain->SetBranchAddress("run_number",&run_number);
   
    chain->SetBranchAddress("PISTA",&PISTA);
    
    TGraph* gel = new TGraph();
    gel = elastic->GetKinematicLine3();

    int nentries = chain->GetEntries();

    hExmean_Tel->Reset();
    hExmean_ThetaLab->Reset();

    double Elab = 0;
    double ThetaLab = 0;
    double Ecal = 0;
    double distance=100;
    
    for(int i=0; i<nentries; i++){
        chain->GetEntry(i);

        //This is because telescope 7 is down
        if(Telescope==8) {Telescope=7;}
        int indice = 3*(Telescope-1); //just start index at 0

        Ecal = parameter[indice] + parameter[indice+1]*Eres + parameter[indice+2]*Eres*Eres;
     

            
        if(Xcalc!=-1000 && Ycalc!=-1000 && Zcalc!=-1000){
            TVector3 PositionOnTarget = TVector3(XTarget+parameter[24-3],YTarget+parameter[25-3],parameter[26-3]);
            TVector3 HitPosition = TVector3(Xcalc,Ycalc,Zcalc);
            TVector3 BeamDirection = TVector3(0,0,1);
            //BeamDirection.RotateX(parameter[27-3]*3.1415/180);
            //BeamDirection.RotateY(parameter[28-3]*3.1415/180);
            TVector3 HitDirection = HitPosition - PositionOnTarget;
            ThetaLab = HitDirection.Angle(BeamDirection);
            
            //Here we do the analysis of the energy loss in the dead layer of the
            //detector.
            //We get the energy of the beam from a document
            //The energy of the carbon is to be reconstructed thanks to the theta
            /*
            double Elab_th = gel->Eval(ThetaLab*180./3.1415);
            double LostE1,LostDE12,LostTarget ;
            double C12E ;
            double ThckAluDE1 = 0.5*micrometer;
            double ThckAluDE2 = 0.5*micrometer;
            double ThckAluE1 = 0.5*micrometer;
            
            double AnglePista = 0; //2 ways of getting it either by doing
                               // Thetapista - ThetaLab
                               // Or using the hitdirection.angle
            */     

            //TVector3 NormalPista ;
            //NormalPista = PISTA->GetDetectorNormal(0) ;
            //AnglePista = (HitDirection.Angle(NormalPista));
           // cout <<AnglePista/deg << " vs " << abs(ThetaLab-BeamDirection.Angle(NormalPista))/deg<< endl;  //This verify if it corrobore with the analysis without information on the target position.
           
            //Analysis for theoretical value
            /* C12E = Elab_th; */
            /* LostTarget = C12CQuick->Eval(C12E,TargetThickness/2,ThetaLab); */
            /* C12E = C12E - LostTarget; */
            /* LostDE12 = C12Al.EnergyLossCalculation(C12E,ThckAluDE1 + ThckAluDE2,AnglePista); */
            /* C12E = C12E - LostDE12; */
            /* LostE1 = C12Al.EnergyLossCalculation(C12E,ThckAluE1,AnglePista); */
            /* C12E = C12E - LostE1; */
            //cout << Elab << " " << Elab_th << endl; 
            
            //Analysis for Ecal
           
            
            Elab = C12AlQuick->Eval(Ecal); // 1 deadlayer
            Elab = C12AlQuick->Eval(Elab); // 2 deadlayer
            Elab += DeltaE;
            Elab = C12AlQuick->Eval(Elab); // 3 deadlayer
            Elab = C12CQuick->Eval(Elab); 
        

            double Ex=-1000;
            /* if(run_number==215) */
            /* { */
            /*     Ex = elasticBe->ReconstructRelativistic(Elab,ThetaLab); */
            /*     if(abs(Ex)<10){ */
            /*         hExBe->Fill(iteration,Ex); */
            /*         hExmeanBe_Tel->Fill(indice/3+1,Ex); */
            /*         hExmeanBe_ThetaLab->Fill(ThetaLab*180/3.1416,Ex); */
            /*     } */
            /* } */
            /* else */
            //{
               
            Ex = elastic->ReconstructRelativistic(Elab,ThetaLab);
            //cout << i << " " << Ex << endl;
            if(abs(Ex)<10){
                hEx->Fill(iteration,Ex);
                hExmean_Tel->Fill(indice/3+1,Ex);
                hExmean_ThetaLab->Fill(ThetaLab*180/3.1416,Ex);



            }
            //}

        }
    }

    double Emean[7];
    double sigma[7];
    double AvEmean=0;
    //distance = 0;  

    for(int i=0;i<7;i++)
    {
        // C12 case
        hExmean = hExmean_Tel->ProjectionY(Form("hExmean_%d",i+1),i+1,i+1);
        Emean[i] = hExmean->GetMean(); // get mean of gaus instead
        double Emin = Emean[i] - 6;
        double Emax = Emean[i] + 6;
        hExmean->GetXaxis()->SetRangeUser(Emin,Emax);
        Emean[i] = hExmean->GetMean();
        sigma[i] = hExmean->GetRMS();
        distance += pow(Emean[i],2) + pow(sigma[i]-0.3,2);
        AvEmean += Emean[i];

        // Be9 case
        /* hExmean = hExmeanBe_Tel->ProjectionY(Form("hExmean_%d",i+1),i+1,i+1); */
        /* Emean[i] = hExmean->GetMean(); */
        /* Emin = Emean[i] - 6; */
        /* Emax = Emean[i] + 1; */
        /* hExmean->GetXaxis()->SetRangeUser(Emin,Emax); */
        /* Emean[i] = hExmean->GetMean(); */
        /* sigma[i] = hExmean->GetRMS(); */
        /* distance += pow(Emean[i],2) + pow(sigma[i]-0.3,2); */

    }
    AvEmean/=8.;
    if(iteration%1==0){
        cout << "Number of iterations: " << iteration  << endl;
        cout << "Emean= " << AvEmean << endl;
        for(int j=0;j<7;j++){
            cout << "Telescope " << j << " : ";
            cout << "  "<< parameter[j*3] << "  " << parameter[j*3+1] << "  " << parameter[j*3+2] <<  endl;
        }
        cout << "Xt Yt Zt :      " << parameter[nb_parameter-1] << "  " << parameter[nb_parameter-2] << "  " << parameter[nb_parameter-3] << endl;
        cout << "distance= " << distance << endl;
    }


    //delete hExmean;
    return distance;
}

